#!/usr/bin/env python3

"""
test.daemontest_d.rpc: RPC unit test for the MMGen suite
"""

import sys, os

from mmgen.cfg import Config
from mmgen.color import yellow, cyan
from mmgen.util import msg, gmsg, make_timestr, pp_fmt, die
from mmgen.protocol import init_proto
from mmgen.rpc import rpc_init
from mmgen.daemon import CoinDaemon
from mmgen.proto.xmr.rpc import MoneroRPCClient, MoneroWalletRPCClient
from mmgen.proto.xmr.daemon import MoneroWalletDaemon

from ..include.common import cfg, qmsg, vmsg, in_nix_environment, test_exec

async def cfg_file_auth_test(cfg, d, bad_auth=False):
	m = 'missing credentials' if bad_auth else f'credentials from {d.cfg_file}'
	qmsg(cyan(f'\n  Testing authentication with {m}:'))
	d.stop()
	d.remove_datadir() # removes cookie file to force authentication from cfg file
	os.makedirs(d.network_datadir)

	if not bad_auth:
		cf = os.path.join(d.datadir, d.cfg_file)
		with open(cf, 'a') as fp:
			fp.write('\nrpcuser = ut_rpc\nrpcpassword = ut_rpc_passw0rd\n')
		d.flag.keep_cfg_file = True

	d.start()

	if bad_auth:
		os.rename(d.auth_cookie_fn, d.auth_cookie_fn+'.bak')
		try:
			await rpc_init(cfg, d.proto)
		except Exception as e:
			vmsg(yellow(str(e)))
		else:
			die(3, 'No error on missing credentials!')
		os.rename(d.auth_cookie_fn+'.bak', d.auth_cookie_fn)
	else:
		rpc = await rpc_init(cfg, d.proto)
		assert rpc.auth.user == 'ut_rpc', f'{rpc.auth.user}: user is not ut_rpc!'

	if not cfg.no_daemon_stop:
		d.stop()
		d.remove_datadir()

async def print_daemon_info(rpc):

	if rpc.proto.base_proto == 'Monero':
		msg(f"""
    DAEMON VERSION: {rpc.daemon_version} [{rpc.daemon_version_str}]
    NETWORK:        {rpc.proto.coin} {rpc.proto.network.upper()}
		""".rstrip())
	else:
		msg(f"""
    DAEMON VERSION: {rpc.daemon_version} [{rpc.daemon_version_str}]
    CAPS:           {rpc.caps}
    NETWORK:        {rpc.proto.coin} {rpc.proto.network.upper()}
    CHAIN:          {rpc.chain}
    BLOCKCOUNT:     {rpc.blockcount}
    CUR_DATE:       {rpc.cur_date} [{make_timestr(rpc.cur_date)}]
		""".rstrip())

	msg(f'    BIND_PORT:      {rpc.daemon.bind_port}')

	if rpc.proto.base_proto == 'Bitcoin':
		def fmt_dict(d):
			return '\n        ' + '\n        '.join(pp_fmt(d).split('\n')) + '\n'
		msg(f"""
    NETWORKINFO:    {fmt_dict(rpc.cached["networkinfo"])}
    BLOCKCHAININFO: {fmt_dict(rpc.cached["blockchaininfo"])}
    DEPLOYMENTINFO: {fmt_dict(rpc.cached["deploymentinfo"])}
    WALLETINFO:     {fmt_dict(await rpc.walletinfo)}
		""".rstrip())

	if rpc.proto.base_proto == 'Ethereum':
		msg(f'    CHAIN_NAMES:    {" ".join(rpc.daemon.proto.chain_names)}')

	msg('')

def do_msg(rpc, backend):
	bname = type(rpc.backend).__name__
	qmsg('  Testing backend {!r}{}'.format(bname, '' if backend == bname else f' [{backend}]'))

class init_test:

	@staticmethod
	async def btc(cfg, daemon, backend, cfg_override):
		rpc = await rpc_init(cfg, daemon.proto, backend=backend, daemon=daemon)
		do_msg(rpc, backend)

		wi = await rpc.walletinfo
		assert wi['walletname'] == cfg_override['btc_tw_name']
		assert wi['walletname'] == rpc.cfg._proto.tw_name, f'{wi["walletname"]!r} != {rpc.cfg._proto.tw_name!r}'
		assert daemon.bind_port == cfg_override['btc_rpc_port']

		bh = (await rpc.call('getblockchaininfo', timeout=300))['bestblockhash']
		await rpc.gathered_call('getblock', ((bh,), (bh, 1)), timeout=300)
		await rpc.gathered_call(None, (('getblock', (bh,)), ('getblock', (bh, 1))), timeout=300)
		return rpc

	@staticmethod
	async def bch(cfg, daemon, backend, cfg_override):
		rpc = await rpc_init(cfg, daemon.proto, backend=backend, daemon=daemon)
		do_msg(rpc, backend)
		return rpc

	ltc = bch

	@staticmethod
	async def eth(cfg, daemon, backend, cfg_override):
		rpc = await rpc_init(cfg, daemon.proto, backend=backend, daemon=daemon)
		do_msg(rpc, backend)
		await rpc.call('eth_blockNumber', timeout=300)
		if rpc.proto.network == 'testnet':
			assert daemon.proto.chain_names == cfg_override['eth_testnet_chain_names']
			assert daemon.bind_port == cfg_override['eth_rpc_port']
		return rpc

	etc = eth

async def run_test(network_ids, test_cf_auth=False, daemon_ids=None, cfg_override=None):

	async def do_test(d, cfg):

		d.wait = True

		if not cfg.no_daemon_stop:
			d.stop()
			d.remove_datadir()

		if not cfg.no_daemon_autostart:
			d.remove_datadir()
			d.start()

		for n, backend in enumerate(cfg._autoset_opts['rpc_backend'].choices):
			test = getattr(init_test, d.proto.coin.lower())
			rpc = await test(cfg, d, backend, cfg_override)
			if not n and cfg.verbose:
				await print_daemon_info(rpc)

		if not cfg.no_daemon_stop:
			d.stop()
			d.remove_datadir()

		if test_cf_auth and sys.platform != 'win32':
			await cfg_file_auth_test(cfg, d)
			await cfg_file_auth_test(cfg, d, bad_auth=True)

		qmsg('')

	my_cfg = Config(cfg_override) if cfg_override else cfg

	for network_id in network_ids:
		proto = init_proto(my_cfg, network_id=network_id)
		all_ids = CoinDaemon.get_daemon_ids(my_cfg, proto.coin)
		ids = set(daemon_ids) & set(all_ids) if daemon_ids else all_ids
		for daemon_id in ids:
			await do_test(CoinDaemon(my_cfg, proto=proto, test_suite=True, daemon_id=daemon_id), my_cfg)

	return True

class unit_tests:

	altcoin_deps = ('ltc', 'bch', 'geth', 'reth', 'erigon', 'parity', 'xmrwallet')
	arm_skip = ('parity',) # no prebuilt binaries for ARM
	riscv_skip = ('parity',) # no prebuilt binaries for RISC-V
	fast_skip = ('reth', 'erigon')

	async def btc(self, name, ut):
		return await run_test(
			['btc', 'btc_tn'],
			test_cf_auth = True,
			cfg_override = {
				'_clone': cfg,
				'btc_rpc_port': 19777,
				'rpc_port':     32323, # ignored
				'btc_tw_name': 'alternate-tracking-wallet',
				'tw_name':     'this-is-overridden',
				'ltc_tw_name': 'this-is-ignored',
				'eth_mainnet_chain_names': ['also', 'ignored'],
		})

	async def ltc(self, name, ut):
		return await run_test(['ltc', 'ltc_tn'], test_cf_auth=True)

	async def bch(self, name, ut):
		return await run_test(['bch', 'bch_tn'], test_cf_auth=True)

	async def geth(self, name, ut):
		# mainnet returns EIP-155 error on empty blockchain:
		return await run_test(
			['eth_tn', 'eth_rt'],
			daemon_ids = ['geth'],
			cfg_override = {
				'_clone': cfg,
				'eth_rpc_port': 19777,
				'rpc_port':     32323, # ignored
				'btc_tw_name': 'ignored',
				'tw_name':     'also-ignored',
				'eth_testnet_chain_names': ['goerli', 'holesky', 'foo', 'bar', 'baz'],
		})

	async def reth(self, name, ut):
		return await run_test(['eth', 'eth_rt'], daemon_ids=['reth']) # TODO: eth_tn

	async def erigon(self, name, ut):
		return await run_test(['eth', 'eth_tn', 'eth_rt'], daemon_ids=['erigon'])

	async def parity(self, name, ut):
		if in_nix_environment() and not test_exec('parity --help'):
			ut.skip_msg('Nix environment')
			return True
		return await run_test(['etc'])

	async def xmrwallet(self, name, ut):

		async def test_monerod_rpc(md):
			rpc = MoneroRPCClient(
				cfg    = cfg,
				proto  = md.proto,
				host   = 'localhost',
				port   = md.rpc_port,
				user   = None,
				passwd = None,
				daemon = md,
			)
			if cfg.verbose:
				await print_daemon_info(rpc)
			rpc.call_raw('get_height')
			rpc.call('get_last_block_header')

		from mmgen.xmrseed import xmrseed

		async def run():
			networks = init_proto(cfg, 'xmr').networks
			daemons = [(
					CoinDaemon(cfg, proto=proto, test_suite=True),
					MoneroWalletDaemon(
						cfg        = cfg,
						proto      = proto,
						test_suite = True,
						wallet_dir = os.path.join('test', 'trash2'),
						datadir    = os.path.join('test', 'trash2', 'wallet_rpc'),
						passwd     = 'ut_rpc_passw0rd')
				) for proto in (init_proto(cfg, 'xmr', network=network) for network in networks)]

			for md, wd in daemons:
				if not cfg.no_daemon_autostart:
					md.start()
				wd.start()

				await test_monerod_rpc(md)

				c = MoneroWalletRPCClient(cfg=cfg, daemon=wd)
				fn = f'monero-{wd.network}-junk-wallet'
				qmsg(f'Creating {wd.network} wallet')
				c.call(
					'restore_deterministic_wallet',
					filename = fn,
					password = 'foo',
					seed     = xmrseed().fromhex('beadface'*8, tostr=True))

				if sys.platform == 'win32':
					wd.stop()
					wd.start()

				qmsg(f'Opening {wd.network} wallet')
				c.call('open_wallet', filename=fn, password='foo')

				await c.stop_daemon()

				if not cfg.no_daemon_stop:
					md.stop()

			gmsg('OK')

		import shutil
		shutil.rmtree('test/trash2', ignore_errors=True)
		os.makedirs('test/trash2/wallet_rpc')
		await run()
		return True



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
# Licensed under the GNU General Public License, Version 3:
#   https://www.gnu.org/licenses
# Public project repositories:
#   https://github.com/mmgen/mmgen-wallet
#   https://gitlab.com/mmgen/mmgen-wallet

"""
wallet.mmgen: MMGen native wallet class
"""

import os

from ..seed import Seed
from ..util import msg, make_timestamp, make_chksum_6, split_into_cols, is_chksum_6
from ..obj import MMGenWalletLabel, get_obj
from ..baseconv import baseconv

from .enc import wallet

class wallet(wallet):

	desc = 'MMGen wallet'

	def __init__(self, *args, **kwargs):
		if self.cfg.label:
			self.label = MMGenWalletLabel(self.cfg.label)
		else:
			self.label = None
		super().__init__(*args, **kwargs)

	# logic identical to _get_hash_preset_from_user()
	def _get_label_from_user(self, *, old_lbl=''):
		prompt = 'Enter a wallet label, or hit ENTER {}: '.format(
			'to reuse the label {}'.format(old_lbl.hl2(encl='‘’')) if old_lbl else
			'for no label')
		from ..ui import line_input
		while True:
			ret = line_input(self.cfg, prompt)
			if ret:
				lbl = get_obj(MMGenWalletLabel, s=ret)
				if lbl:
					return lbl
				else:
					msg('Invalid label.  Trying again...')
			else:
				return old_lbl or MMGenWalletLabel('No Label')

	# logic identical to _get_hash_preset()
	def _get_label(self):
		if hasattr(self, 'ss_in') and hasattr(self.ss_in.ssdata, 'label'):
			old_lbl = self.ss_in.ssdata.label
			if self.cfg.keep_label:
				lbl = old_lbl
				self.cfg._util.qmsg('Reusing label {} at user request'.format(lbl.hl2(encl='‘’')))
			elif self.label:
				lbl = self.label
				self.cfg._util.qmsg('Using user-configured label {}'.format(lbl.hl2(encl='‘’')))
			else: # Prompt, using old value as default
				lbl = self._get_label_from_user(old_lbl=old_lbl)
			if (not self.cfg.keep_label) and self.op == 'pwchg_new':
				self.cfg._util.qmsg('Label {}'.format('unchanged' if lbl == old_lbl else f'changed to {lbl!r}'))
		elif self.label:
			lbl = self.label
			self.cfg._util.qmsg('Using user-configured label {}'.format(lbl.hl2(encl='‘’')))
		else:
			lbl = self._get_label_from_user()
		self.ssdata.label = lbl

	def _encrypt(self):
		self._get_first_pw_and_hp_and_encrypt_seed()
		self._get_label()
		d = self.ssdata
		d.pw_status = ('NE', 'E')[len(d.passwd)==0]
		d.timestamp = make_timestamp()

	def _format(self):
		d = self.ssdata
		s = self.seed
		bc = baseconv('b58')
		slt_fmt  = bc.frombytes(d.salt, pad='seed', tostr=True)
		es_fmt = bc.frombytes(d.enc_seed, pad='seed', tostr=True)
		lines = (
			d.label,
			'{} {} {} {} {}'.format(s.sid.lower(), d.key_id.lower(), s.bitlen, d.pw_status, d.timestamp),
			'{}: {} {} {}'.format(d.hash_preset, *self.crypto.get_hash_params(d.hash_preset)),
			'{} {}'.format(make_chksum_6(slt_fmt), split_into_cols(4, slt_fmt)),
			'{} {}'.format(make_chksum_6(es_fmt),  split_into_cols(4, es_fmt))
		)
		chksum = make_chksum_6(' '.join(lines).encode())
		self.fmt_data = '\n'.join((chksum,)+lines) + '\n'

	def _deformat(self):

		def check_master_chksum(lines, desc):

			if len(lines) != 6:
				msg(f'Invalid number of lines ({len(lines)}) in {desc} data')
				return False

			if not is_chksum_6(lines[0]):
				msg(f'Incorrect master checksum ({lines[0]}) in {desc} data')
				return False

			chk = make_chksum_6(' '.join(lines[1:]))
			if not self.cfg._util.compare_chksums(lines[0], 'master', chk, 'computed',
						hdr='For wallet master checksum', verbose=True):
				return False

			return True

		lines = self.fmt_data.splitlines()
		if not check_master_chksum(lines, self.desc):
			return False

		d = self.ssdata
		d.label = MMGenWalletLabel(lines[1])

		d1, d2, d3, d4, d5 = lines[2].split()
		d.seed_id = d1.upper()
		d.key_id  = d2.upper()
		self.check_usr_seed_len(bitlen=int(d3))
		d.pw_status, d.timestamp = d4, d5

		hpdata = lines[3].split()

		d.hash_preset = hp = hpdata[0][:-1]  # a string!
		self.cfg._util.qmsg(f'Hash preset of wallet: {hp!r}')
		if self.cfg.hash_preset and self.cfg.hash_preset != hp:
			self.cfg._util.qmsg(f'Warning: ignoring user-requested hash preset {self.cfg.hash_preset!r}')

		hash_params = tuple(map(int, hpdata[1:]))

		if hash_params != self.crypto.get_hash_params(d.hash_preset):
			msg(f'Hash parameters {" ".join(hash_params)!r} don’t match hash preset {d.hash_preset!r}')
			return False

		lmin, _, lmax = sorted(baseconv('b58').seedlen_map_rev) # 22, 33, 44
		for i, key in (4, 'salt'), (5, 'enc_seed'):
			l = lines[i].split(' ')
			chk = l.pop(0)
			b58_val = ''.join(l)

			if len(b58_val) < lmin or len(b58_val) > lmax:
				msg(f'Invalid format for {key} in {self.desc}: {l}')
				return False

			if not self.cfg._util.compare_chksums(
					chk,
					key,
					make_chksum_6(b58_val),
					'computed checksum',
					verbose = True):
				return False

			val = baseconv('b58').tobytes(b58_val, pad='seed')
			if val is False:
				msg(f'Invalid base 58 number: {b58_val}')
				return False

			setattr(d, key, val)

		return True

	def _decrypt(self):
		d = self.ssdata
		# Needed for multiple transactions with {}-txsign
		d.passwd = self._get_passphrase(
			add_desc = os.path.basename(self.infile.name) if self.cfg.quiet else '')
		key = self.crypto.make_key(d.passwd, d.salt, d.hash_preset)
		ret = self.crypto.decrypt_seed(d.enc_seed, key, seed_id=d.seed_id, key_id=d.key_id)
		if ret:
			self.seed = Seed(self.cfg, seed_bin=ret)
			return True
		else:
			return False

	def _filename(self):
		s = self.seed
		d = self.ssdata
		return '{}-{}[{},{}].{}'.format(
			s.fn_stem,
			d.key_id,
			s.bitlen,
			d.hash_preset,
			self.ext)



#!/usr/bin/env python3

"""
test.modtest_d.misc: utility unit tests for the MMGen suite
"""

from mmgen.color import cyan
from mmgen.util import fmt_list, fmt_dict, list_gen

from ..include.common import vmsg

class unit_tests:

	def fmt_list(self, name, ut):

		samples = {
			'pids': [18234, 18444, 19324],
			'vardata': [None, True, 1234, 'sample string'],
		}
		chks = {
			'vardata': {
				None:        "'None', 'True', '1234', 'sample string'",
				'dfl':       "'None', 'True', '1234', 'sample string'",
				'utf8':      "“None”, “True”, “1234”, “sample string”",
				'bare':      "None True 1234 'sample string'",
				'no_quotes': "None, True, 1234, sample string",
				'no_spc':    "'None','True','1234','sample string'",
				'min':       "None,True,1234,sample string",
				'repr':      "None, True, 1234, 'sample string'",
				'csv':       "None,True,1234,'sample string'",
				'col':       "    + None\n    + True\n    + 1234\n    + sample string",
			}

		}

		col1_w = max(len(str(e)) for e in list(chks.values())[0]) + 1

		for _name, sample in samples.items():
			vmsg(cyan(f'Input: {sample}'))
			for fmt in list(chks.values())[0]:
				spc = '\n' if fmt in ('col', 'list') else ' '
				indent = '    + ' if fmt == 'col' else ''
				res = fmt_list(sample, fmt=fmt, indent=indent) if fmt else fmt_list(sample, indent=indent)
				vmsg(f'  {str(fmt)+":":{col1_w}}{spc}{res}')
				if _name in chks:
					assert res == chks[_name][fmt], f'{res} != {chks[_name][fmt]}'

		vmsg('')
		return True

	def fmt_dict(self, name, ut):
		samples = {
			'url': {
				'name': 'Example',
				'desc': 'Sample URL',
				'rank': 1,
				'error': None,
				'url': 'https://example.com/foobar.html',
			},
			'choice': {
				'c': 'curl',
				'a': 'aiohttp',
				'r': 'requests',
			},
			'cmdline': {
				'cmd': ['ls', '-l'],
				'text': 'foo bar',
				'stdin': None,
				'offset': 123,
				'env': {},
			}
		}
		chks = {
		'cmdline': {
			None:           "'cmd' (['ls', '-l']), 'text' (foo bar), 'stdin' (None), 'offset' (123), 'env' ({})",
			'dfl':          "'cmd' (['ls', '-l']), 'text' (foo bar), 'stdin' (None), 'offset' (123), 'env' ({})",
			'square':       "'cmd' [['ls', '-l']], 'text' [foo bar], 'stdin' [None], 'offset' [123], 'env' [{}]",
			'equal':        "'cmd'=['ls', '-l'], 'text'=foo bar, 'stdin'=None, 'offset'=123, 'env'={}",
			'equal_spaced': "'cmd' = ['ls', '-l'], 'text' = foo bar, 'stdin' = None, 'offset' = 123, 'env' = {}",
			'kwargs':       "cmd=['ls', '-l'], text='foo bar', stdin=None, offset=123, env={}",
			'colon':        "cmd:['ls', '-l'], text:'foo bar', stdin:None, offset:123, env:{}",
		}
		}

		col1_w = max(len(str(e)) for e in list(chks.values())[0]) + 1

		for _name, sample in samples.items():
			vmsg(cyan(f'Input: {sample}'))
			for fmt in list(chks.values())[0]:
				res = fmt_dict(sample, fmt=fmt) if fmt else fmt_dict(sample)
				vmsg(f'  {str(fmt)+":":{col1_w}} {res}')
				if _name in chks:
					assert res == chks[_name][fmt], f'{res} != {chks[_name][fmt]}'

		vmsg('')
		return True

	def list_gen(self, name, ut):
		res = list_gen(
			['a'],
			['b', False],
			['c', 'x'],
			['d', int],
			['e', None, 1, 'f', isinstance(7, int)],
			['g', 'h', 0],
			[None],
			[0],
			[False],
		)
		chk = ['a', 'c', 'd', 'e', None, 1, 'f', None, 0, False]

		vmsg('=> ' + str(res))
		assert res == chk, f'{res} != {chk}'
		vmsg('')
		return True



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
# Licensed under the GNU General Public License, Version 3:
#   https://www.gnu.org/licenses
# Public project repositories:
#   https://github.com/mmgen/mmgen-wallet
#   https://gitlab.com/mmgen/mmgen-wallet

"""
proto.btc.tx.info: Bitcoin transaction info class
"""

from ....tx.info import TxInfo
from ....util import fmt, die
from ....color import red, green, blue, pink
from ....addr import MMGenID

class TxInfo(TxInfo):

	sort_orders = ('addr', 'raw')
	txinfo_hdr_fs = '{hdr}\n  ID={i} ({a} {c}) RBF={r} Sig={s} Locktime={l}\n'
	txinfo_hdr_fs_short = 'TX {i} ({a} {c}) RBF={r} Sig={s} Locktime={l}\n'
	txinfo_ftr_fs = fmt("""
		Input amount: {i} {d}
		Spend amount: {s} {d}
		Change:       {C} {d}
		Fee:          {a} {c}{r}
	""")

	def format_rel_fee(self):
		tx = self.tx
		return ' ({} {}, {} of spend amount)'.format(
			pink(tx.fee_abs2rel(tx.fee)),
			tx.rel_fee_disp,
			pink('{:0.6f}%'.format(tx.fee / tx.send_amt * 100))
		)

	def format_abs_fee(self, iwidth, /, *, color=None):
		return self.tx.fee.fmt(iwidth, color=color)

	def format_verbose_footer(self):
		tx = self.tx
		tsize = len(tx.serialized) // 2 if tx.serialized else 'unknown'
		out = f'Transaction size: Vsize {tx.estimate_size()} (estimated), Total {tsize}'
		if tx.name in ('Signed', 'OnlineSigned'):
			wsize = tx.deserialized.witness_size
			out += f', Base {tsize-wsize}, Witness {wsize}'
		return out + '\n'

	def format_body(self, blockcount, nonmm_str, max_mmwid, enl, *, terse, sort):

		if sort not in self.sort_orders:
			die(1, '{!r}: invalid transaction view sort order. Valid options: {}'.format(
				sort,
				','.join(self.sort_orders)))

		def get_mmid_fmt(e, is_input):
			if e.mmid:
				return e.mmid.fmt2(
					max_mmwid,
					encl = '()',
					color = True,
					append_chars = ('', ' (chg)')[bool(not is_input and e.is_chg and terse)],
					append_color = 'green')
			else:
				return MMGenID.fmtc(
					'[vault address]' if not is_input and e.is_vault else nonmm_str,
					max_mmwid,
					color = True)

		def format_io(desc):
			io = getattr(tx, desc)
			is_input = desc == 'inputs'
			yield desc.capitalize() + ':\n' + enl
			confs_per_day = 60*60*24 // tx.proto.avg_bdi

			io_sorted = {
				'addr': lambda: sorted(
					io, # prepend '+' (sorts before '0') to ensure non-MMGen addrs are displayed first
					key = lambda o: (o.mmid.sort_key if o.mmid else f'+{o.addr}') + f'{o.amt:040.20f}'),
				'raw':  lambda: io
			}[sort]

			def data_disp(data):
				return f'OP_RETURN data ({len(data)} bytes)'

			if terse:
				iwidth = max(len(str(int(e.amt))) for e in io)
				addr_w = max((len(e.addr.views[vp1]) if e.addr else len(data_disp(e.data))) for f in (tx.inputs, tx.outputs) for e in f)
				for n, e in enumerate(io_sorted()):
					yield '{:3} {} {} {} {}\n'.format(
						n+1,
						e.addr.fmt(vp1, addr_w, color=True) if e.addr else blue(data_disp(e.data).ljust(addr_w)),
						get_mmid_fmt(e, is_input) if e.addr else ''.ljust(max_mmwid),
						e.amt.fmt(iwidth, color=True),
						tx.dcoin)
					if have_bch and e.addr:
						yield '{:3} [{}]\n'.format('', e.addr.hl(vp2, color=False))
			else:
				col1_w = len(str(len(io))) + 1
				for n, e in enumerate(io_sorted()):
					mmid_fmt = get_mmid_fmt(e, is_input)
					if is_input and blockcount:
						confs = e.confs + blockcount - tx.blockcount
						days = int(confs // confs_per_day)
					def gen():
						if is_input:
							yield (n+1, 'tx,vout:', f'{e.txid.hl()},{red(str(e.vout))}')
							yield ('',  'address:', f'{e.addr.hl(vp1)} {mmid_fmt}')
							if have_bch:
								yield ('', '', f'[{e.addr.hl(vp2, color=False)}]')
						else:
							yield (
								n + 1,
								'address:',
								(f'{e.addr.hl(vp1)} {mmid_fmt}' if e.addr else e.data.hl(add_label=True)))
							if have_bch and e.addr:
								yield ('', '', f'[{e.addr.hl(vp2, color=False)}]')
						if e.comment:
							yield ('',  'comment:', e.comment.hl())
						yield     ('',  'amount:',  f'{e.amt.hl()} {tx.dcoin}')
						if is_input and blockcount:
							yield ('',  'confirmations:', f'{confs} (around {days} days)')
						if not is_input and e.is_chg:
							yield ('',  'change:',  green('True'))
					yield '\n'.join('{:>{w}} {:<8} {}'.format(*d, w=col1_w) for d in gen()) + '\n\n'

		tx = self.tx

		if self.cfg._proto.coin == 'BCH':
			have_bch = True
			vp1 = 1 if not self.cfg.cashaddr else not self.cfg._proto.cashaddr
			vp2 = (vp1 + 1) % 2
		else:
			have_bch = False
			vp1 = 0

		return (
			'Inputs/Outputs sort order: {}'.format({
				'raw':  pink('UNSORTED'),
				'addr': pink('ADDRESS')
			}[sort])
			+ ('\n\n', '\n')[terse]
			+ ''.join(format_io('inputs'))
			+ ''.join(format_io('outputs')))

	def strfmt_locktime(self, locktime=None, *, terse=False):
		# Locktime itself is an unsigned 4-byte integer which can be parsed two ways:
		#
		# If less than 500 million, locktime is parsed as a block height. The transaction can be
		# added to any block which has this height or higher.
		# MMGen note: s/this height or higher/a higher block height/
		#
		# If greater than or equal to 500 million, locktime is parsed using the Unix epoch time
		# format (the number of seconds elapsed since 1970-01-01T00:00 UTC). The transaction can be
		# added to any block whose block time is greater than the locktime.
		num = locktime or self.tx.locktime
		if num is None:
			return '(None)'
		elif num.bit_length() > 32:
			die(2, f'{num!r}: invalid nLockTime value (integer size greater than 4 bytes)!')
		elif num >= 500_000_000:
			import time
			return ' '.join(time.strftime('%c', time.gmtime(num)).split()[1:])
		elif num > 0:
			return '{}{}'.format(('block height ', '')[terse], num)
		else:
			die(2, f'{num!r}: invalid nLockTime value!')



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
opts: command-line options processing for the MMGen Project
"""

import sys, os, re
from collections import namedtuple

from .cfg import gc

def negated_opts(opts, data={}):
	if data:
		return data
	else:
		data.update(dict(
			((k[3:] if k.startswith('no-') else f'no-{k}'), v)
				for k, v in opts.items()
					if len(k) > 1 and not v.has_parm))
		return data

def get_opt_by_substring(opt, opts):
	matches = [o for o in opts if o.startswith(opt)]
	if len(matches) == 1:
		return matches[0]
	if len(matches) > 1:
		from .util import die
		die('CmdlineOptError', f'--{opt}: ambiguous option (not unique substring)')

def process_uopts(cfg, opts_data, opts, need_proto):

	from .util import die

	def get_uopts():
		nonlocal uargs
		idx = 1
		argv_len = len(sys.argv)
		while idx < argv_len:
			arg = sys.argv[idx]
			if len(arg) > 4096:
				raise RuntimeError(f'{len(arg)} bytes: command-line argument too long')
			if arg.startswith('--'):
				if len(arg) == 2:
					uargs = sys.argv[idx+1:]
					return
				opt, parm = arg[2:].split('=', 1) if '=' in arg else (arg[2:], None)
				if len(opt) < 2:
					die('CmdlineOptError', f'--{opt}: option name must be at least two characters long')
				if (
						(_opt := opt) in opts
						or (_opt := get_opt_by_substring(_opt, opts))):
					if opts[_opt].has_parm:
						if parm:
							yield (opts[_opt].name, parm)
						else:
							idx += 1
							if idx == argv_len or (parm := sys.argv[idx]).startswith('-'):
								die('CmdlineOptError', f'missing parameter for option --{_opt}')
							yield (opts[_opt].name, parm)
					else:
						if parm:
							die('CmdlineOptError', f'option --{_opt} requires no parameter')
						yield (opts[_opt].name, True)
				elif (
						(_opt := opt) in negated_opts(opts)
						or (_opt := get_opt_by_substring(_opt, negated_opts(opts)))):
					if parm:
						die('CmdlineOptError', f'option --{_opt} requires no parameter')
					yield (negated_opts(opts)[_opt].name, False)
				elif (
						need_proto
						and (not gc.cmd_caps or gc.cmd_caps.rpc)
						and any(opt.startswith(coin + '-') for coin in gc.rpc_coins)):
					opt_name = opt.replace('-', '_')
					from .protocol import init_proto
					try:
						refval = init_proto(cfg, opt.split('-', 1)[0], return_cls=True).get_opt_clsval(cfg, opt_name)
					except AttributeError:
						die('CmdlineOptError', f'--{opt}: unrecognized option')
					else:
						if refval is None: # None == no parm
							if parm:
								die('CmdlineOptError', f'option --{opt} requires no parameter')
							yield (opt_name, True)
						else:
							from .cfg import conv_type
							if parm:
								yield (opt_name,
									conv_type(opt_name, parm, refval, src='cmdline'))
							else:
								idx += 1
								if idx == argv_len or (parm := sys.argv[idx]).startswith('-'):
									die('CmdlineOptError', f'missing parameter for option --{opt}')
								yield (opt_name,
									conv_type(opt_name, parm, refval, src='cmdline'))
				else:
					die('CmdlineOptError', f'--{opt}: unrecognized option')
			elif arg[0] == '-' and len(arg) > 1:
				for j, sopt in enumerate(arg[1:], 2):
					if sopt in opts:
						if opts[sopt].has_parm:
							if arg[j:]:
								yield (opts[sopt].name, arg[j:])
							else:
								idx += 1
								if idx == argv_len or (parm := sys.argv[idx]).startswith('-'):
									die('CmdlineOptError', f'missing parameter for option -{sopt}')
								yield (opts[sopt].name, parm)
							break
						else:
							yield (opts[sopt].name, True)
					else:
						die('CmdlineOptError', f'-{sopt}: unrecognized option')
			else:
				uargs = sys.argv[idx:]
				return
			idx += 1

	uargs = []
	uopts = dict(get_uopts())

	return uopts, uargs

cmd_opts_v1_pat      = re.compile(r'^-([a-zA-Z0-9-]), --([a-zA-Z0-9-]{2,64})(=| )(.+)')

cmd_opts_v2_pat      = re.compile(r'^\t\t\t(.)(.) -([a-zA-Z0-9-]), --([a-z0-9-]{2,64})(=| )(.+)')
cmd_opts_v2_help_pat = re.compile(r'^\t\t\t(.)(.) (?:-([a-zA-Z0-9-]), --([a-z0-9-]{2,64})(=| ))?(.+)')

global_opts_pat      = re.compile(r'^\t\t\t(.)(.) --([a-z0-9-]{2,64})(=| )(.+)')
global_opts_help_pat = re.compile(r'^\t\t\t(.)(.) (?:--([{}a-zA-Z0-9-]{2,64})(=| ))?(.+)')

opt_tuple = namedtuple('cmdline_option', ['name', 'has_parm'])

def parse_opts(cfg, opts_data, global_opts_data, global_filter_codes, *, need_proto):

	def parse_v1():
		for line in opts_data['text']['options'].strip().splitlines():
			if m := cmd_opts_v1_pat.match(line):
				ret = opt_tuple(m[2].replace('-', '_'), m[3] == '=')
				yield (m[1], ret)
				yield (m[2], ret)

	def parse_v2():
		cmd_filter_codes = opts_data['filter_codes']
		coin_codes = global_filter_codes.coin
		for line in opts_data['text']['options'].splitlines():
			m = cmd_opts_v2_pat.match(line)
			if m and (coin_codes is None or m[1] in coin_codes) and m[2] in cmd_filter_codes:
				ret = opt_tuple(m[4].replace('-', '_'), m[5] == '=')
				yield (m[3], ret)
				yield (m[4], ret)

	def parse_global():
		coin_codes = global_filter_codes.coin
		cmd_codes = global_filter_codes.cmd
		for line in global_opts_data['text']['options'].splitlines():
			m = global_opts_pat.match(line)
			if m and (
					(coin_codes is None or m[1] in coin_codes) and
					(cmd_codes is None or m[2] in cmd_codes)):
				yield (m[3], opt_tuple(m[3].replace('-', '_'), m[4] == '='))

	opts = tuple((parse_v2 if 'filter_codes' in opts_data else parse_v1)()) + tuple(parse_global())

	uopts, uargs = process_uopts(cfg, opts_data, dict(opts), need_proto)

	return namedtuple('parsed_cmd_opts', ['user_opts', 'cmd_args', 'opts'])(
		uopts, # dict
		uargs, # list, callers can pop
		tuple(v.name for k, v in opts if len(k) > 1)
	)

def opt_preproc_debug(po):
	d = (
		('Cmdline',            ' '.join(sys.argv), False),
		('Filtered opts',      po.filtered_opts,   False),
		('User-selected opts', po.user_opts,       False),
		('Cmd args',           po.cmd_args,        False),
		('Opts',               po.opts,            True),
	)
	from .util import Msg, fmt_list
	Msg('\n=== opts.py debug ===')
	for label, data, pretty in d:
		Msg('    {:<20}: {}'.format(label, '\n' + fmt_list(data, fmt='col', indent=' '*8) if pretty else data))

opts_data_dfl = {
	'text': {
		'desc': '',
		'usage':'[options]',
		'options': """
-h, --help         Print this help message
--, --longhelp     Print help message for long (global) options
"""
	}
}

def get_coin():
	for n, arg in enumerate(sys.argv[1:]):
		if len(arg) > 4096:
			raise RuntimeError(f'{len(arg)} bytes: command-line argument too long')
		if arg.startswith('--coin='):
			return arg.removeprefix('--coin=').lower()
		if arg == '--coin':
			if len(sys.argv) < n + 3:
				from .util import die
				die('CmdlineOptError', f'{arg}: missing parameter')
			return sys.argv[n + 2].lower()
		if arg == '-' or not arg.startswith('-'): # stop at first non-option
			return 'btc'
	return 'btc'

class Opts:

	def __init__(
			self,
			cfg,
			*,
			opts_data,
			init_opts,    # dict containing opts to pre-initialize
			parsed_opts,
			need_proto):

		if len(sys.argv) > 257:
			raise RuntimeError(f'{len(sys.argv) - 1}: too many command-line arguments')

		opts_data = opts_data or opts_data_dfl

		self.global_filter_codes = self.get_global_filter_codes(need_proto)
		self.opts_data = opts_data

		po = parsed_opts or parse_opts(
			cfg,
			opts_data,
			self.global_opts_data,
			self.global_filter_codes,
			need_proto = need_proto)

		cfg._args = po.cmd_args
		cfg._uopts = uopts = po.user_opts

		if init_opts: # initialize user opts to given value
			for uopt, val in init_opts.items():
				if uopt not in uopts:
					uopts[uopt] = val

		cfg._opts = self
		cfg._parsed_opts = po
		cfg._use_env = True
		cfg._use_cfg_file = not 'skip_cfg_file' in uopts

		# Make these available to usage():
		cfg._usage_data = opts_data['text'].get('usage2') or opts_data['text']['usage']
		cfg._usage_code = opts_data.get('code', {}).get('usage')
		cfg._help_pkg = self.help_pkg

		if os.getenv('MMGEN_DEBUG_OPTS'):
			opt_preproc_debug(po)

		for funcname in self.info_funcs:
			if funcname in uopts:
				import importlib
				getattr(importlib.import_module(self.help_pkg), funcname)(cfg) # exits

class UserOpts(Opts):

	help_pkg = 'mmgen.help'
	info_funcs = ('version', 'show_hash_presets', 'list_daemon_ids')

	global_opts_data = {
		#  coin code : cmd code : opt : opt param : text
		'text': {
			'options': """
			-- --accept-defaults      Accept defaults at all prompts
			hp --cashaddr=0|1         Display addresses in cashaddr format (default: 1)
			-c --coin=c               Choose coin unit. Default: BTC. Current choice: {cu_dfl}
			er --token=t              Specify an ERC20 token by address or symbol
			-- --color=0|1            Disable or enable color output (default: 1)
			-- --columns=N            Force N columns of output with certain commands
			Rr --scroll               Use the curses-like scrolling interface for
			+                         tracking wallet views
			-- --force-256-color      Force 256-color output when color is enabled
			-- --pager                Pipe output of certain commands to pager (WIP)
			-- --data-dir=path        Specify {pnm} data directory location
			rr --daemon-data-dir=path Specify coin daemon data directory location
			Rr --daemon-id=ID         Specify the coin daemon ID
			rr --ignore-daemon-version Ignore coin daemon version check
			Rr --list-daemon-ids      List all available daemon IDs
			xr --http-timeout=t       Set HTTP timeout in seconds for JSON-RPC connections
			-- --no-license           Suppress the GPL license prompt
			Rr --rpc-host=HOST        Communicate with coin daemon running on host HOST
			rr --rpc-port=PORT        Communicate with coin daemon listening on port PORT
			br --rpc-user=USER        Authenticate to coin daemon using username USER
			br --rpc-password=PASS    Authenticate to coin daemon using password PASS
			Rr --rpc-backend=backend  Use backend 'backend' for JSON-RPC communications
			Rr --aiohttp-rpc-queue-len=N Use N simultaneous RPC connections with aiohttp
			-p --regtest=0|1          Disable or enable regtest mode
			-- --testnet=0|1          Disable or enable testnet
			-- --test-suite           Use test suite configuration
			br --tw-name=NAME         Specify alternate name for the BTC/LTC/BCH tracking
			+                         wallet (default: ‘{tw_name}’)
			-- --skip-cfg-file        Skip reading the configuration file
			-- --version              Print version information and exit
			-- --usage                Print usage information and exit
			b- --bob                  Specify user ‘Bob’ in MMGen regtest mode
			b- --alice                Specify user ‘Alice’ in MMGen regtest mode
			b- --carol                Specify user ‘Carol’ in MMGen regtest mode
			rr COIN-SPECIFIC OPTIONS:
			rr   For descriptions, refer to the non-prefixed versions of these options above
			rr   Prefixed options override their non-prefixed counterparts
			rr   OPTION                            SUPPORTED PREFIXES
			Rr --PREFIX-daemon-id                btc ltc bch eth etc
			rr --PREFIX-ignore-daemon-version    btc ltc bch eth etc xmr
			br --PREFIX-tw-name                  btc ltc bch
			Rr --PREFIX-rpc-host                 btc ltc bch eth etc
			rr --PREFIX-rpc-port                 btc ltc bch eth etc xmr
			br --PREFIX-rpc-user                 btc ltc bch
			br --PREFIX-rpc-password             btc ltc bch
			Rr --PREFIX-max-tx-fee               btc ltc bch eth etc
			Rr PROTO-SPECIFIC OPTIONS:
			Rr   Option                            Supported Prefixes
			Rr --PREFIX-chain-names              eth-mainnet eth-testnet etc-mainnet etc-testnet
			""",
		},
		'code': {
			'options': lambda proto, help_notes, s: s.format(
				pnm     = gc.proj_name,
				cu_dfl  = proto.coin,
				tw_name = help_notes('dfl_twname')),
		}
	}

	@staticmethod
	def get_global_filter_codes(need_proto):
		"""
		Enable options based on the value of --coin and name of executable

		Both must produce a matching code list, or None, for the option to be enabled

		Coin codes:
		  'b' - Bitcoin or Bitcoin code fork supporting RPC
		  'R' - Bitcoin or Ethereum code fork supporting RPC
		  'e' - Ethereum or Ethereum code fork
		  'h' - Bitcoin Cash
		  'r' - local RPC coin
		  'X' - remote RPC coin
		  'x' - local or remote RPC coin
		  '-' - any coin
		Cmd codes:
		  'p' - proto required
		  'c' - proto required, --coin recognized
		  'r' - RPC required
		  '-' - no capabilities required
		"""
		ret = namedtuple('global_filter_codes', ['coin', 'cmd'])
		if caps := gc.cmd_caps:
			coin = get_coin() if caps.use_coin_opt else None
			# a return value of None removes the filter, enabling all options for the given criterion
			return ret(
				coin = caps.coin_codes or (
					None if coin is None else
					['-', 'r', 'R', 'b', 'h', 'x'] if coin == 'bch' else
					['-', 'r', 'R', 'b', 'x'] if coin in gc.btc_fork_rpc_coins else
					['-', 'r', 'R', 'e', 'x'] if coin in gc.eth_fork_coins else
					['-', 'r', 'x'] if coin in gc.local_rpc_coins else
					['-', 'X', 'x'] if coin in gc.remote_rpc_coins else
					['-']),
				cmd = (
					['-']
					+ (['r'] if caps.rpc else [])
					+ (['p', 'c'] if caps.proto and caps.use_coin_opt else ['p'] if caps.proto else [])
				))
		else: # unmanaged command: enable everything
			return ret(None, None)



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
# Licensed under the GNU General Public License, Version 3:
#   https://www.gnu.org/licenses
# Public project repositories:
#   https://github.com/mmgen/mmgen-wallet
#   https://gitlab.com/mmgen/mmgen-wallet

"""
proto.eth.tx.online: Ethereum online signed transaction class
"""

from ....util import msg, die
from ....color import yellow, green, orange
from ....tx import online as TxBase
from .. import erigon_sleep
from .signed import Signed, TokenSigned

class OnlineSigned(Signed, TxBase.OnlineSigned):

	async def test_sendable(self, txhex):
		raise NotImplementedError('transaction testing not implemented for Ethereum')

	async def send_checks(self):
		self.check_correct_chain()
		if not self.disable_fee_check and (self.fee > self.proto.max_tx_fee):
			die(2, 'Transaction fee ({}) greater than {} max_tx_fee ({} {})!'.format(
				self.fee,
				self.proto.name,
				self.proto.max_tx_fee,
				self.proto.coin))
		await self.status.display()

	async def send_with_node(self, txhex):
		try:
			ret = await self.rpc.call('eth_sendRawTransaction', '0x' + txhex)
		except Exception as e:
			msg(orange('\n'+str(e)))
			die(2, f'Send of MMGen transaction {self.txid} failed')
		await erigon_sleep(self)
		return ret.removeprefix('0x')

	async def post_network_send(self, coin_txid):
		res = await self.get_receipt(coin_txid)
		if not res:
			if self.cfg.wait:
				msg('{} {} {}'.format(
					yellow('Send of'),
					coin_txid.hl(),
					yellow('failed? (failed to get receipt)')))
			return False
		msg(f'Gas sent: {res.gas_sent.hl()}\n'
			f'Gas used: {res.gas_used.hl()}')
		if res.status == 0:
			if res.gas_used == res.gas_sent:
				msg(yellow('All gas was used!'))
			msg('{} {} {}'.format(
				yellow('Send of'),
				coin_txid.hl(),
				yellow('failed (status=0)')))
			return False
		msg(f'Status: {green(str(res.status))}')
		if res.contract_addr:
			msg('Contract address: {}'.format(res.contract_addr.hl(0)))
		return True

class TokenOnlineSigned(TokenSigned, OnlineSigned):

	def parse_txfile_serialized_data(self):
		from ....addr import ContractAddr
		from ..contract import Token
		OnlineSigned.parse_txfile_serialized_data(self)
		o = self.txobj
		assert self.twctl.token == o['to']
		o['token_addr'] = ContractAddr(self.proto, o['to'])
		o['decimals']   = self.twctl.decimals
		t = Token(self.cfg, self.proto, o['token_addr'], decimals=o['decimals'])
		o['amt'] = t.transferdata2amt(o['data'])
		o['token_to'] = t.transferdata2sendaddr(o['data'])
		if self.is_swap:
			from .transaction import Transaction
			from .. import rlp
			etx = rlp.decode(bytes.fromhex(self.serialized2), Transaction)
			d = etx.to_dict()
			o['router_gas'] = d['startgas']

class Sent(TxBase.Sent, OnlineSigned):
	pass

class TokenSent(TxBase.Sent, TokenOnlineSigned):
	pass

class AutomountOnlineSigned(TxBase.AutomountOnlineSigned, OnlineSigned):
	pass

class AutomountSent(TxBase.AutomountSent, AutomountOnlineSigned):
	pass

class TokenAutomountOnlineSigned(TxBase.AutomountOnlineSigned, TokenOnlineSigned):
	pass

class TokenAutomountSent(TxBase.AutomountSent, TokenAutomountOnlineSigned):
	pass



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
# Licensed under the GNU General Public License, Version 3:
#   https://www.gnu.org/licenses
# Public project repositories:
#   https://github.com/mmgen/mmgen-wallet
#   https://gitlab.com/mmgen/mmgen-wallet

"""
wallet.enc: encrypted wallet base class
"""

from ..cfg import gc
from ..util import msg, make_chksum_8, die
from .base import wallet

class wallet(wallet):

	def __init__(self, *args, **kwargs):
		from ..crypto import Crypto
		self.crypto = Crypto(self.cfg)
		super().__init__(*args, **kwargs)

	def _decrypt_retry(self):
		while True:
			if self._decrypt():
				break
			if self.passwd_file:
				die(2, 'Passphrase from password file, so exiting')
			msg('Trying again...')

	def _get_hash_preset_from_user(self, old_preset, *, add_desc=''):
		prompt = 'Enter {}hash preset for {}{}{},\nor hit ENTER to {} value ({!r}): '.format(
			('old ' if self.op=='pwchg_old' else 'new ' if self.op=='pwchg_new' else ''),
			('', 'new ')[self.op=='new'],
			self.desc,
			('', ' '+add_desc)[bool(add_desc)],
			('accept the default', 'reuse the old')[self.op=='pwchg_new'],
			old_preset)
		return self.crypto.get_hash_preset_from_user(old_preset=old_preset, prompt=prompt)

	def _get_hash_preset(self, *, add_desc=''):
		if hasattr(self, 'ss_in') and hasattr(self.ss_in.ssdata, 'hash_preset'):
			old_hp = self.ss_in.ssdata.hash_preset
			if self.cfg.keep_hash_preset:
				hp = old_hp
				self.cfg._util.qmsg(f'Reusing hash preset {hp!r} at user request')
			elif self.cfg.hash_preset:
				hp = self.cfg.hash_preset
				self.cfg._util.qmsg(f'Using user-configured hash preset {hp!r}')
			else: # Prompt, using old value as default
				hp = self._get_hash_preset_from_user(old_preset=old_hp, add_desc=add_desc)
			if (not self.cfg.keep_hash_preset) and self.op == 'pwchg_new':
				self.cfg._util.qmsg('Hash preset {}'.format(
					'unchanged' if hp == old_hp else f'changed to {hp!r}'))
		elif self.cfg.hash_preset:
			hp = self.cfg.hash_preset
			self.cfg._util.qmsg(f'Using user-configured hash preset {hp!r}')
		else:
			hp = self._get_hash_preset_from_user(
				old_preset = gc.dfl_hash_preset,
				add_desc   = add_desc)
		self.ssdata.hash_preset = hp

	def _get_new_passphrase(self):
		return self.crypto.get_new_passphrase(
			data_desc = ('new ' if self.op in ('new', 'conv') else '') + self.desc,
			hash_preset = self.ssdata.hash_preset,
			passwd_file = self.passwd_file,
			pw_desc = ('new ' if self.op=='pwchg_new' else '') + 'passphrase')

	def _get_passphrase(self, *, add_desc=''):
		return self.crypto.get_passphrase(
			data_desc = self.desc + (f' {add_desc}' if add_desc else ''),
			passwd_file = self.passwd_file,
			pw_desc = ('old ' if self.op == 'pwchg_old' else '') + 'passphrase')

	def _get_first_pw_and_hp_and_encrypt_seed(self):
		d = self.ssdata
		self._get_hash_preset()

		if hasattr(self, 'ss_in') and hasattr(self.ss_in.ssdata, 'passwd'):
			old_pw = self.ss_in.ssdata.passwd
			if self.cfg.keep_passphrase:
				d.passwd = old_pw
				self.cfg._util.qmsg('Reusing passphrase at user request')
			else:
				d.passwd = self._get_new_passphrase()
				if self.op == 'pwchg_new':
					self.cfg._util.qmsg('Passphrase {}'.format('unchanged' if d.passwd == old_pw else 'changed'))
		else:
			d.passwd = self._get_new_passphrase()

		from hashlib import sha256
		d.salt     = sha256(self.crypto.get_random(128)).digest()[:self.crypto.salt_len]
		key        = self.crypto.make_key(d.passwd, d.salt, d.hash_preset)
		d.key_id   = make_chksum_8(key)
		d.enc_seed = self.crypto.encrypt_seed(self.seed.data, key)



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
#
# Project source code repository: https://github.com/mmgen/mmgen-wallet
# Licensed according to the terms of GPL Version 3.  See LICENSE for details.

"""
test.cmdtest_d.tool: tool tests for the MMGen cmdtest.py test suite
"""

import sys, os

from mmgen.util import suf
from mmgen.color import cyan

from ..include.common import (
	vmsg,
	read_from_file,
	write_to_file,
	cmp_or_die,
	joinpath,
	getrand
)
from .include.common import hincog_fn, incog_id_fn, hincog_offset, tool_enc_passwd, ref_dir
from .base import CmdTestBase
from .main import CmdTestMain

class CmdTestTool(CmdTestMain, CmdTestBase):
	"interactive 'mmgen-tool' commands"
	networks = ('btc',)
	segwit_opts_ok = False
	tmpdir_nums = [9]
	enc_infn = 'tool_encrypt.in'
	cmd_group = (
		('tool_find_incog_data',
			(9, '‘mmgen-tool find_incog_data’', [[[hincog_fn], 1], [[incog_id_fn], 1]])
		),
		('tool_rand2file',
			(9, '‘mmgen-tool rand2file’', [])
		),
		('tool_encrypt',
			(9, '‘mmgen-tool encrypt’ (random data)', [])
		),
		('tool_decrypt',
			(9, '‘mmgen-tool decrypt’ (random data)', [[[enc_infn+'.mmenc'], 9]])
		),
		('tool_twview_bad_comment',
			(9, '‘mmgen-tool twview’ (with bad comment)', [])
		),
		('tool_decrypt_keystore',
			(9, '‘mmgen-tool decrypt_keystore’', [])
		),
		('tool_decrypt_geth_keystore',
			(9, '‘mmgen-tool decrypt_geth_keystore’', [])
		),
		('tool_api',
			(9, 'tool API (initialization, config methods, wif2addr)', [])
		),
		# ('tool_encrypt_ref', (9, '‘mmgen-tool encrypt’ (reference text)', [])),
	)

	def tool_rand2file(self):
		from mmgen.util2 import parse_bytespec
		for nbytes in ('1', '1023', '1K', '1048575', '1M', '1048577', '123M'):
			t = self.spawn(
				'mmgen-tool',
				['-d', self.tmpdir, '-r0', 'rand2file', 'rand2file.out', nbytes],
				extra_desc='({} byte{})'.format(nbytes, suf(parse_bytespec(nbytes)))
			)
			t.expect('random data written to file')
			t.read()
			t.p.wait()
			t.ok()
		t.skip_ok = True
		return t

	def tool_encrypt(self):
		infile = joinpath(self.tmpdir, self.enc_infn)
		write_to_file(infile, getrand(1033), binary=True)
		t = self.spawn('mmgen-tool', ['-d', self.tmpdir, self.usr_rand_arg, 'encrypt', infile])
		t.usr_rand(self.usr_rand_chars)
		t.hash_preset('data', '1')
		t.passphrase_new('data', tool_enc_passwd)
		t.written_to_file('Encrypted data')
		return t

	def tool_decrypt(self, f1):
		out_fn = 'tool_encrypt.out'
		t = self.spawn('mmgen-tool', ['-d', self.tmpdir, 'decrypt', f1, 'outfile='+out_fn, 'hash_preset=1'])
		t.passphrase('data', tool_enc_passwd)
		t.written_to_file('Decrypted data')
		d1 = self.read_from_tmpfile(self.enc_infn, binary=True)
		d2 = self.read_from_tmpfile(out_fn, binary=True)
		cmp_or_die(d1, d2)
		return t

	def tool_find_incog_data(self, f1, f2):
		i_id = read_from_file(f2).rstrip()
		vmsg(f'Incog ID: {cyan(i_id)}')
		t = self.spawn('mmgen-tool', ['-d', self.tmpdir, 'find_incog_data', f1, i_id])
		o = t.expect_getend(f'Incog data for ID {i_id} found at offset ')
		if not sys.platform == 'win32':
			os.unlink(f1) # causes problems with MSYS2
		cmp_or_die(hincog_offset, int(o))
		return t

	def tool_twview_bad_comment(self): # test correct operation of get_tw_label()
		t = self.spawn(
			'mmgen-tool',
			['twview'],
			env = {'MMGEN_BOGUS_UNSPENT_DATA': joinpath(ref_dir, 'bad-comment-unspent.json')},
			exit_val = 2)
		t.expect('cannot be converted to TwComment')
		return t

	def _decrypt_keystore(self, cmd, fn, pw, chk):
		if self.cfg.no_altcoin:
			return 'skip'
		t = self.spawn('mmgen-tool', ['-d', self.tmpdir, cmd, fn])
		t.expect('Enter passphrase: ', pw+'\n')
		t.expect(chk)
		return t

	def tool_decrypt_keystore(self):
		return self._decrypt_keystore(
			cmd = 'decrypt_keystore',
			fn  = 'test/ref/altcoin/98831F3A-keystore-wallet.json',
			pw = 'abc',
			chk = read_from_file('test/ref/98831F3A.bip39').strip())

	def tool_decrypt_geth_keystore(self):
		return self._decrypt_keystore(
			cmd = 'decrypt_geth_keystore',
			fn  = 'test/ref/ethereum/geth-wallet.json',
			pw  = '',
			chk = '9627ddb68354f5e0ff45fb2da49d7a20a013b7257a83ef4adbbbd87aeaccc75e')

	def tool_api(self):
		t = self.spawn(
				'tool_api_test.py',
				(['no_altcoin'] if self.cfg.no_altcoin else []),
				cmd_dir = 'test/misc')
		t.expect('legacy.*compressed.*segwit.*bech32', regex=True)
		return t



#!/usr/bin/env python3

"""
test.daemontest_d.tx: TX daemon tests for the MMGen suite
"""

import json

from mmgen.color import purple, cyan
from mmgen.util import Msg, Msg_r
from mmgen.devtools import Pmsg
from mmgen.protocol import init_proto
from mmgen.tx import CompletedTX
from mmgen.proto.btc.tx.base import DeserializeTX
from mmgen.rpc import rpc_init
from mmgen.daemon import CoinDaemon
from mmgen.tx import NewTX

from ..include.common import cfg, start_test_daemons, stop_test_daemons, qmsg

def print_info(name, extra_desc):
	if cfg.names:
		Msg_r('{} {} {}'.format(
			purple('Testing'),
			cyan(f'{name} ({extra_desc})'),
			'' if cfg.quiet else '\n'))
	else:
		Msg_r(f'Testing {extra_desc}')
		if not cfg.quiet:
			Msg('')

async def test_tx(tx_proto, tx_hex, desc, n):

	def has_nonstandard_outputs(outputs):
		for o in outputs:
			t = o['scriptPubKey']['type']
			if t in ('nonstandard', 'pubkey', 'nulldata'):
				return True
		return False

	rpc = await rpc_init(cfg, proto=tx_proto, ignore_wallet=True)
	d = await rpc.call('decoderawtransaction', tx_hex)

	if has_nonstandard_outputs(d['vout']):
		return False

	dt = DeserializeTX(tx_proto, tx_hex)

	if cfg.verbose:
		Msg('\n\n================================ Core vector: ==================================')
	Msg_r('.' if cfg.quiet else f'{n:>3}) {desc}\n')
	if cfg.verbose:
		Pmsg(d)
		Msg('\n------------------------------ MMGen deserialized: -----------------------------')
		Pmsg(dt._asdict())

	# metadata
	assert dt.txid == d['txid'], 'TXID does not match'
	assert dt.locktime == d['locktime'], 'Locktime does not match'
	assert dt.version == d['version'], 'Version does not match'

	# inputs
	a, b = d['vin'], dt.txins
	for i in range(len(a)):
		assert a[i]['txid'] == b[i]['txid'], f'TxID of input {i} does not match'
		assert a[i]['vout'] == b[i]['vout'], f'vout of input {i} does not match'
		assert a[i]['sequence'] == int(b[i]['nSeq'], 16), (
			f'nSeq of input {i} does not match')
		if 'txinwitness' in a[i]:
			assert a[i]['txinwitness'] == b[i]['witness'], (
				f'witness of input {i} does not match')

	# outputs
	a, b = d['vout'], dt.txouts
	for i in range(len(a)):
		if 'addresses' in a[i]['scriptPubKey']:
			A = a[i]['scriptPubKey']['addresses'][0]
			B = b[i]['addr']
			fs = 'address of output {} does not match\nA: {}\nB: {}'
			assert A == B, fs.format(i, A, B)

		A = tx_proto.coin_amt(a[i]['value'])
		B = b[i]['amt']
		fs = 'value of output {} does not match\nA: {}\nB: {}'
		assert A == B, fs.format(i, A, B)

		A = a[i]['scriptPubKey']['hex']
		B = b[i]['scriptPubKey']
		fs = 'scriptPubKey of output {} does not match\nA: {}\nB: {}'
		assert A == B, fs.format(i, A, B)

async def do_mmgen_ref(daemons, fns, name, desc):
	# NB: remove_datadir is required here for some reason (seems to be Bitcoin Core version-dependent)
	start_test_daemons(*daemons, remove_datadir=True)
	print_info(name, desc)
	for n, fn in enumerate(fns):
		tx = await CompletedTX(cfg=cfg, filename=fn, quiet_open=True)
		await test_tx(
			tx_proto = tx.proto,
			tx_hex   = tx.serialized,
			desc     = fn,
			n        = n + 1)
	stop_test_daemons(*daemons, remove_datadir=True)
	Msg('OK')
	return True

class unit_tests:

	altcoin_deps = ('mmgen_ref_alt',)

	async def newtx(self, name, ut):
		qmsg('  Testing NewTX initializer')
		d = CoinDaemon(cfg, network_id='btc', test_suite=True)
		d.start()

		proto = init_proto(cfg, 'btc', need_amt=True)
		NewTX(cfg=cfg, proto=proto, target='tx')

		d.stop()
		d.remove_datadir()
		qmsg('  OK')
		return True

	async def core_vectors(self, name, ut):

		start_test_daemons('btc')

		with open('test/ref/tx_valid.json') as fp:
			core_data = json.loads(fp.read())

		print_info(name, 'Bitcoin Core test vectors')

		n = 1
		desc = '(no description)'
		for e in core_data:
			if isinstance(e[0], list):
				await test_tx(
					tx_proto = init_proto(cfg, 'btc', need_amt=True),
					tx_hex   = e[1],
					desc     = desc,
					n        = n)
				n += 1
			else:
				desc = e[0]

		Msg('OK')
		stop_test_daemons('btc', remove_datadir=True)
		return True

	async def mmgen_ref(self, name, ut):
		return await do_mmgen_ref(
			('btc', 'btc_tn'),
			(
				'test/ref/0B8D5A[15.31789,14,tl=1320969600].rawtx',
				'test/ref/0C7115[15.86255,14,tl=1320969600].testnet.rawtx',
				'test/ref/542169[5.68152,34].sigtx',
			),
			name,
			'MMGen reference transactions [Bitcoin]')

	async def mmgen_ref_alt(self, name, ut):
		return await do_mmgen_ref(
			('ltc', 'ltc_tn', 'bch'),
			(
				'test/ref/litecoin/AF3CDF-LTC[620.76194,1453,tl=1320969600].rawtx',
				'test/ref/litecoin/A5A1E0-LTC[1454.64322,1453,tl=1320969600].testnet.rawtx',
				'test/ref/460D4D-BCH[10.19764,tl=1320969600].rawtx'
			),
			name,
			'MMGen reference transactions [Altcoin]')



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
test.cmdtest_d.seedsplit: Seed split/join tests for the cmdtest.py test suite
"""

import os

from mmgen.wallet import get_wallet_cls
from mmgen.util import capfirst

from ..include.common import strip_ansi_escapes, cmp_or_die
from .include.common import get_file_with_ext
from .base import CmdTestBase

ref_wf = 'test/ref/98831F3A.bip39'
ref_sid = '98831F3A'
wpasswd = 'abc'
sh1_passwd = 'xyz'
dfl_wcls = get_wallet_cls('mmgen')

class CmdTestSeedSplit(CmdTestBase):
	'splitting and joining seeds'
	networks = ('btc',)
	tmpdir_nums = [23]
	color = True
	cmd_group = (
		('ss_walletgen',             'wallet generation'),
		('ss_2way_A_dfl1',           '2-way seed split (share A)'),
		('ss_2way_B_dfl1',           '2-way seed split (share B)'),
		('ss_2way_join_dfl1',        '2-way seed join'),
		('ss_2way_A_dfl2',           '2-way seed split ‘default’ (share A)'),
		('ss_2way_B_dfl2',           '2-way seed split ‘default’ (share B)'),
		('ss_2way_join_dfl2',        '2-way seed join ‘default’'),
		('ss_2way_A_alice',          '2-way seed split ‘alice’ (share A)'),
		('ss_2way_B_alice',          '2-way seed split ‘alice’ (share B)'),
		('ss_2way_join_alice',       '2-way seed join ‘alice’'),
		('ss_2way_join_alice_mix',   '2-way seed join ‘alice’ (out of order)'),
		('ss_2way_A_dfl_master3',    '2-way seed split with master share #3 (share A)'),
		('ss_2way_B_dfl_master3',    '2-way seed split with master share #3 (share B)'),
		('ss_2way_join_dfl_master3', '2-way seed join with master share #3'),
		('ss_2way_A_dfl_usw',        '2-way seed split of user-specified wallet (share A)'),
		('ss_2way_B_dfl_usw',        '2-way seed split of user-specified wallet (share B)'),
		('ss_2way_join_dfl_usw',     '2-way seed join of user-specified wallet'),
		('ss_3way_A_dfl',            '3-way seed split (share A)'),
		('ss_3way_B_dfl',            '3-way seed split (share B)'),
		('ss_3way_C_dfl',            '3-way seed split (share C)'),
		('ss_3way_join_dfl',         '3-way seed join'),
		('ss_3way_join_dfl_mix',     '3-way seed join (out of order)'),
		('ss_3way_A_foobar_master7', '3-way seed split ‘φυβαρ’ with master share #7 (share A)'),
		('ss_3way_B_foobar_master7', '3-way seed split ‘φυβαρ’ with master share #7 (share B)'),
		('ss_3way_C_foobar_master7', '3-way seed split ‘φυβαρ’ with master share #7 (share C)'),
		('ss_3way_join_foobar_master7', '3-way seed join ‘φυβαρ’ with master share #7'),
		('ss_3way_join_foobar_master7_mix', '3-way seed join ‘φυβαρ’ with master share #7 (out of order)'),

		('ss_3way_join_dfl_bad_invocation', 'bad invocation of ‘mmgen-seedjoin’ - --id-str with non-master join'),
		('ss_bad_invocation1',       'bad invocation of ‘mmgen-seedsplit’ - no arguments'),
		('ss_bad_invocation2',       'bad invocation of ‘mmgen-seedsplit’ - master share with split specifier'),
		('ss_bad_invocation3',       'bad invocation of ‘mmgen-seedsplit’ - nonexistent file'),
		('ss_bad_invocation4',       'bad invocation of ‘mmgen-seedsplit’ - invalid file extension'),
		('ss_bad_invocation5',       'bad invocation of ‘mmgen-seedjoin’ - no arguments'),
		('ss_bad_invocation6',       'bad invocation of ‘mmgen-seedjoin’ - one file argument'),
		('ss_bad_invocation7',       'bad invocation of ‘mmgen-seedjoin’ - invalid file extension'),
		('ss_bad_invocation8',       'bad invocation of ‘mmgen-seedjoin’ - nonexistent file'),
		('ss_bad_invocation9',       'bad invocation of ‘mmgen-seedsplit’ - bad specifier'),
		('ss_bad_invocation10',      'bad invocation of ‘mmgen-seedsplit’ - nonexistent file'),
		('ss_bad_invocation11',      'bad invocation of ‘mmgen-seedsplit’ - invalid file extension'),
	)

	def get_tmp_subdir(self, subdir):
		return os.path.join(self.tmpdir, subdir)

	def ss_walletgen(self):
		t = self.spawn('mmgen-walletgen', ['-r0', '-p1'])
		t.passphrase_new('new '+dfl_wcls.desc, wpasswd)
		t.label()
		self.write_to_tmpfile('dfl.sid', strip_ansi_escapes(t.expect_getend('Seed ID: ')))
		t.expect('move it to the data directory? (Y/n): ', 'y')
		t.written_to_file(capfirst(dfl_wcls.desc))
		return t

	def ss_splt(self, tdir, ofmt, spec, add_args=[], wf=None, master=None):
		try:
			os.mkdir(self.get_tmp_subdir(tdir))
		except:
			pass
		t = self.spawn('mmgen-seedsplit',
				['-q', '-d', self.get_tmp_subdir(tdir), '-r0', '-o', ofmt]
				+ (['-L', (spec or 'label')] if ofmt == 'w' else [])
				+ add_args
				+ ([f'--master-share={master}'] if master else [])
				+ ([wf] if wf else [])
				+ ([spec] if spec else []))
		if not wf:
			t.passphrase(dfl_wcls.desc, wpasswd)
		if spec:
			from mmgen.seedsplit import SeedSplitSpecifier
			sss = SeedSplitSpecifier(spec)
			pat = rf'Processing .*\b{sss.idx}\b of \b{sss.count}\b of .* id .*‘{sss.id}’'
		else:
			pat = f'master share #{master}'
		t.expect(pat, regex=True)
		ocls = get_wallet_cls(fmt_code=ofmt)
		if ocls.enc:
			t.hash_preset('new '+ocls.desc, '1')
			t.passphrase_new('new '+ocls.desc, sh1_passwd)
			if ocls.type == 'incog_hidden':
				t.hincog_create(1234)
		t.written_to_file(capfirst(ocls.desc))
		return t

	def ss_join(
			self,
			tdir,
			ofmt,
			in_exts,
			add_args = [],
			sid      = None,
			exit_val = None,
			master   = None,
			id_str   = None):
		td = self.get_tmp_subdir(tdir)
		shares = [get_file_with_ext(td, f) for f in in_exts]
		if not sid:
			sid = self.read_from_tmpfile('dfl.sid')
		t = self.spawn('mmgen-seedjoin',
				add_args
				+ ([f'--master-share={master}'] if master else [])
				+ ([f'--id-str={id_str}'] if id_str else [])
				+ ['-d', td, '-o', ofmt]
				+ (['--label', 'Joined Wallet Label', '-r0'] if ofmt == 'w' else [])
				+ shares,
				exit_val = exit_val)
		if exit_val:
			return t
		icls = (dfl_wcls if 'mmdat' in in_exts
			else get_wallet_cls('incog') if 'mmincog' in in_exts
			else get_wallet_cls('incog_hex') if 'mmincox' in in_exts
			else get_wallet_cls('incog_hidden') if '-H' in add_args
			else None)
		if icls.type.startswith('incog'):
			t.hash_preset(icls.desc, '1')
		if icls:
			t.passphrase(icls.desc, sh1_passwd)
		if master:
			fs = "master share #{}, split id.*‘{}’.*, share count {}"
			pat = fs.format(
				master,
				id_str or 'default',
				len(shares) + (icls.type=='incog_hidden'))
			t.expect(pat, regex=True)
		sid_cmp = strip_ansi_escapes(t.expect_getend('Joined Seed ID: '))
		cmp_or_die(sid, sid_cmp)
		ocls = get_wallet_cls(fmt_code=ofmt)
		if ocls.type == 'mmgen':
			t.hash_preset('new '+ocls.desc, '1')
			t.passphrase_new('new '+ocls.desc, wpasswd)
		t.written_to_file(capfirst(ocls.desc))
		return t

	def get_hincog_arg(self, tdir, suf='-default-2of2'):
		sid = self.read_from_tmpfile('dfl.sid')
		return os.path.join(self.tmpdir, tdir, sid+suf+'.hincog') + ',123'

	def ss_2way_A_dfl1(self):
		return self.ss_splt('2way_dfl1', 'w', '1:2')
	def ss_2way_B_dfl1(self):
		return self.ss_splt('2way_dfl1', 'bip39', '2:2')
	def ss_2way_join_dfl1(self):
		return self.ss_join('2way_dfl1', 'w', ['mmdat', 'bip39'])

	def ss_2way_A_dfl2(self):
		return self.ss_splt('2way_dfl2', 'seed', 'default:1:2')
	def ss_2way_B_dfl2(self):
		return self.ss_splt('2way_dfl2', 'hincog', 'default:2:2', ['-J', self.get_hincog_arg('2way_dfl2')])
	def ss_2way_join_dfl2(self):
		return self.ss_join('2way_dfl2', 'mmhex', ['mmseed'], ['-H', self.get_hincog_arg('2way_dfl2')])

	def ss_2way_A_alice(self):
		return self.ss_splt('2way_alice', 'w', 'alice:1:2')
	def ss_2way_B_alice(self):
		return self.ss_splt('2way_alice', 'mmhex', 'alice:2:2')
	def ss_2way_join_alice(self):
		return self.ss_join('2way_alice', 'seed', ['mmdat', 'mmhex'])
	def ss_2way_join_alice_mix(self):
		return self.ss_join('2way_alice', 'seed', ['mmhex', 'mmdat'])

	def ss_2way_A_dfl_usw(self):
		return self.ss_splt('2way_dfl_usw', 'words', '1:2', [], wf=ref_wf)
	def ss_2way_B_dfl_usw(self):
		return self.ss_splt('2way_dfl_usw', 'incog', '2:2', [], wf=ref_wf)
	def ss_2way_join_dfl_usw(self):
		return self.ss_join('2way_dfl_usw', 'mmhex', ['mmwords', 'mmincog'], sid=ref_sid)

	def ss_3way_A_dfl(self):
		return self.ss_splt('3way_dfl', 'words', '1:3')
	def ss_3way_B_dfl(self):
		return self.ss_splt('3way_dfl', 'incog_hex', '2:3')
	def ss_3way_C_dfl(self):
		return self.ss_splt('3way_dfl', 'bip39', '3:3')
	def ss_3way_join_dfl(self):
		return self.ss_join('3way_dfl', 'mmhex', ['mmwords', 'mmincox', 'bip39'])
	def ss_3way_join_dfl_mix(self):
		return self.ss_join('3way_dfl', 'mmhex', ['bip39', 'mmwords', 'mmincox'])

	def ss_2way_A_dfl_master3(self):
		return self.ss_splt('2way_dfl_master3', 'w', '', master=3)
	def ss_2way_B_dfl_master3(self):
		return self.ss_splt('2way_dfl_master3', 'bip39', '2:2', master=3)
	def ss_2way_join_dfl_master3(self):
		return self.ss_join('2way_dfl_master3', 'mmhex', ['mmdat', 'bip39'], master=3)

	tdir2 = '3way_foobar_master7'
	def ss_3way_C_foobar_master7(self):
		return self.ss_splt(self.tdir2, 'hincog', '',
					['-J', self.get_hincog_arg(self.tdir2, '-master7')], master=7)
	def ss_3way_B_foobar_master7(self):
		return self.ss_splt(self.tdir2, 'bip39', 'φυβαρ:2:3', master=7)
	def ss_3way_A_foobar_master7(self):
		return self.ss_splt(self.tdir2, 'mmhex', 'φυβαρ:3:3', master=7)
	def ss_3way_join_foobar_master7(self):
		return self.ss_join(self.tdir2, 'seed', ['bip39', 'mmhex'],
							['-H', self.get_hincog_arg(self.tdir2, '-master7')], master=7, id_str='φυβαρ')
	def ss_3way_join_foobar_master7_mix(self):
		return self.ss_join(self.tdir2, 'seed', ['mmhex', 'bip39'],
							['-H', self.get_hincog_arg(self.tdir2, '-master7')], master=7, id_str='φυβαρ')

	def ss_bad_invocation(self, cmd, args, exit_val, errmsg):
		t = self.spawn(cmd, args, exit_val=exit_val)
		t.expect(errmsg, regex=True)
		return t

	def ss_3way_join_dfl_bad_invocation(self):
		t = self.ss_join('3way_dfl', 'mmhex',
			['mmwords', 'mmincox', 'bip39'],
			id_str   = 'foo',
			exit_val = 1)
		t.expect('option meaningless')
		return t

	def ss_bad_invocation1(self):
		return self.ss_bad_invocation(
			'mmgen-seedsplit', [], 1, 'USAGE:')

	def ss_bad_invocation2(self):
		return self.ss_bad_invocation(
			'mmgen-seedsplit', ['-M1', '1:9'], 1, 'meaningless in master share context')

	def ss_bad_invocation3(self):
		return self.ss_bad_invocation(
			'mmgen-seedsplit', [self.tmpdir+'/no.mmdat', '1:9'], 1, 'input file .* not found')

	def ss_bad_invocation4(self):
		return self.ss_bad_invocation(
			'mmgen-seedsplit', [self.tmpdir+'/dfl.sid', '1:9'], 1, 'unrecognized .* extension')

	def ss_bad_invocation5(self):
		return self.ss_bad_invocation(
			'mmgen-seedjoin', [], 1, 'USAGE:')

	def ss_bad_invocation6(self):
		return self.ss_bad_invocation(
			'mmgen-seedjoin', [self.tmpdir+'/a'], 1, 'USAGE:')

	def ss_bad_invocation7(self):
		return self.ss_bad_invocation(
			'mmgen-seedjoin', [self.tmpdir+'/a', self.tmpdir+'/b'], 1, 'unrecognized .* extension')

	def ss_bad_invocation8(self):
		return self.ss_bad_invocation(
			'mmgen-seedjoin', [self.tmpdir+'/a.mmdat', self.tmpdir+'/b.mmdat'], 1, 'input file .* not found')

	def ss_bad_invocation9(self):
		return self.ss_bad_invocation(
			'mmgen-seedsplit', ['x'], 1, 'USAGE:')

	def ss_bad_invocation10(self):
		return self.ss_bad_invocation(
			'mmgen-seedsplit', [self.tmpdir+'/a.mmdat', '1:2'], 1, 'input file .* not found')

	def ss_bad_invocation11(self):
		return self.ss_bad_invocation(
			'mmgen-seedsplit', [self.tmpdir+'/dfl.sid', '1:2'], 1, 'unrecognized .* extension')



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
tool.util: Utility commands for the 'mmgen-tool' utility
"""

import sys, os

from .common import tool_cmd_base

class tool_cmd(tool_cmd_base):
	"general string conversion and hashing utilities"

	# mmgen.util2.bytespec_map
	def bytespec(self, dd_style_byte_specifier: str):
		"""
		convert a byte specifier such as ‘4GB’ into an integer

		Valid specifiers:

		  c  = 1
		  w  = 2
		  b  = 512
		  kB = 1000
		  K  = 1024
		  MB = 1000000
		  M  = 1048576
		  GB = 1000000000
		  G  = 1073741824
		  TB = 1000000000000
		  T  = 1099511627776
		  PB = 1000000000000000
		  P  = 1125899906842624
		  EB = 1000000000000000000
		  E  = 1152921504606846976
		"""
		from ..util2 import parse_bytespec
		return parse_bytespec(dd_style_byte_specifier)

	# mmgen.util2.bytespec_map
	def to_bytespec(self,
			n: int,
			dd_style_byte_specifier: str,
			*,
			fmt:       'width and precision of output' = '0.2',
			print_sym: 'print the specifier after the numerical value' = True,
			strip:     'strip trailing zeroes' = False,
			add_space: 'with print_sym, add space between value and specifier' = False):
		"""
		convert an integer to a byte specifier such as ‘4GB’

		Supported specifiers:

		  c  = 1
		  w  = 2
		  b  = 512
		  kB = 1000
		  K  = 1024
		  MB = 1000000
		  M  = 1048576
		  GB = 1000000000
		  G  = 1073741824
		  TB = 1000000000000
		  T  = 1099511627776
		  PB = 1000000000000000
		  P  = 1125899906842624
		  EB = 1000000000000000000
		  E  = 1152921504606846976
		"""
		from ..util2 import int2bytespec
		return int2bytespec(
			n,
			dd_style_byte_specifier,
			fmt,
			print_sym = print_sym,
			strip     = strip,
			add_space = add_space)

	def randhex(self,
			nbytes: 'number of bytes to output' = 32):
		"print 'n' bytes (default 32) of random data in hex format"
		from ..crypto import Crypto
		return Crypto(self.cfg).get_random(nbytes).hex()

	def hexreverse(self, hexstr: 'sstr'):
		"reverse bytes of a hexadecimal string"
		return bytes.fromhex(hexstr.strip())[::-1].hex()

	def hexlify(self, infile: str):
		"convert bytes in file to hexadecimal (use '-' for stdin)"
		from ..fileutil import get_data_from_file
		data = get_data_from_file(self.cfg, infile, dash=True, quiet=True, binary=True)
		return data.hex()

	def unhexlify(self, hexstr: 'sstr'):
		"convert a hexadecimal string to bytes (warning: outputs binary data)"
		return bytes.fromhex(hexstr)

	def hexdump(self,
			infile: str,
			*,
			cols:      'number of columns in output' = 8,
			line_nums: "format for line numbers (valid choices: 'hex','dec')" = 'hex'):
		"create hexdump of data from file (use '-' for stdin)"
		from ..fileutil import get_data_from_file
		from ..util2 import pretty_hexdump
		data = get_data_from_file(self.cfg, infile, dash=True, quiet=True, binary=True)
		return pretty_hexdump(data, cols=cols, line_nums=line_nums).rstrip()

	def unhexdump(self, infile: str):
		"decode hexdump from file (use '-' for stdin) (warning: outputs binary data)"
		if sys.platform == 'win32':
			import msvcrt
			msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
		from ..fileutil import get_data_from_file
		from ..util2 import decode_pretty_hexdump
		hexdata = get_data_from_file(self.cfg, infile, dash=True, quiet=True)
		return decode_pretty_hexdump(hexdata)

	def hash160(self, hexstr: 'sstr'):
		"compute ripemd160(sha256(data)) (convert hex pubkey to hex addr)"
		from ..proto.btc.common import hash160
		return hash160(bytes.fromhex(hexstr)).hex()

	# TODO: handle stdin
	def hash256(self,
			data: str,
			*,
			file_input: 'first arg is the name of a file containing the data' = False,
			hex_input:  'first arg is a hexadecimal string' = False):
		"compute sha256(sha256(data)) (double sha256)"
		from hashlib import sha256
		if file_input:
			from ..fileutil import get_data_from_file
			b = get_data_from_file(self.cfg, data, binary=True)
		elif hex_input:
			from ..util2 import decode_pretty_hexdump
			b = decode_pretty_hexdump(data)
		else:
			b = data
		return sha256(sha256(b.encode()).digest()).hexdigest()

	def id6(self, infile: str):
		"generate 6-character MMGen ID for a file (use '-' for stdin)"
		from ..util import make_chksum_6
		from ..fileutil import get_data_from_file
		return make_chksum_6(
			get_data_from_file(self.cfg, infile, dash=True, quiet=True, binary=True))

	def str2id6(self, string: 'sstr'): # retain ignoring of space for backwards compat
		"generate 6-character MMGen ID for a string, ignoring spaces in string"
		from ..util import make_chksum_6
		return make_chksum_6(''.join(string.split()))

	def id8(self, infile: str):
		"generate 8-character MMGen ID for a file (use '-' for stdin)"
		from ..util import make_chksum_8
		from ..fileutil import get_data_from_file
		return make_chksum_8(
			get_data_from_file(self.cfg, infile, dash=True, quiet=True, binary=True))

	def randb58(self, *,
			nbytes: 'number of bytes to output' = 32,
			pad:    'pad output to this width' = 0):
		"generate random data (default: 32 bytes) and convert it to base 58"
		from ..baseconv import baseconv
		from ..crypto import Crypto
		return baseconv('b58').frombytes(Crypto(self.cfg).get_random(nbytes), pad=pad, tostr=True)

	def bytestob58(self, infile: str, *, pad: 'pad output to this width' = 0):
		"convert bytes to base 58 (supply data via STDIN)"
		from ..fileutil import get_data_from_file
		from ..baseconv import baseconv
		data = get_data_from_file(self.cfg, infile, dash=True, quiet=True, binary=True)
		return baseconv('b58').frombytes(data, pad=pad, tostr=True)

	def b58tobytes(self, b58_str: 'sstr', *, pad: 'pad output to this width' = 0):
		"convert a base 58 string to bytes (warning: outputs binary data)"
		from ..baseconv import baseconv
		return baseconv('b58').tobytes(b58_str, pad=pad)

	def hextob58(self, hexstr: 'sstr', *, pad: 'pad output to this width' = 0):
		"convert a hexadecimal string to base 58"
		from ..baseconv import baseconv
		return baseconv('b58').fromhex(hexstr, pad=pad, tostr=True)

	def b58tohex(self, b58_str: 'sstr', *, pad: 'pad output to this width' = 0):
		"convert a base 58 string to hexadecimal"
		from ..baseconv import baseconv
		return baseconv('b58').tohex(b58_str, pad=pad)

	def hextob58chk(self, hexstr: 'sstr'):
		"convert a hexadecimal string to base58-check encoding"
		from ..proto.btc.common import b58chk_encode
		return b58chk_encode(bytes.fromhex(hexstr))

	def b58chktohex(self, b58chk_str: 'sstr'):
		"convert a base58-check encoded string to hexadecimal"
		from ..proto.btc.common import b58chk_decode
		return b58chk_decode(b58chk_str).hex()

	def hextob32(self, hexstr: 'sstr', *, pad: 'pad output to this width' = 0):
		"convert a hexadecimal string to an MMGen-flavor base 32 string"
		from ..baseconv import baseconv
		return baseconv('b32').fromhex(hexstr, pad=pad, tostr=True)

	def b32tohex(self, b32_str: 'sstr', *, pad: 'pad output to this width' = 0):
		"convert an MMGen-flavor base 32 string to hexadecimal"
		from ..baseconv import baseconv
		return baseconv('b32').tohex(b32_str.upper(), pad=pad)

	def hextob6d(self,
			hexstr: 'sstr',
			*,
			pad: 'pad output to this width' = 0,
			add_spaces: 'add a space after every 5th character' = True):
		"convert a hexadecimal string to die roll base6 (base6d)"
		from ..baseconv import baseconv
		from ..util2 import block_format
		ret = baseconv('b6d').fromhex(hexstr, pad=pad, tostr=True)
		return block_format(ret, gw=5, cols=None).strip() if add_spaces else ret

	def b6dtohex(self, b6d_str: 'sstr', *, pad: 'pad output to this width' = 0):
		"convert a die roll base6 (base6d) string to hexadecimal"
		from ..baseconv import baseconv
		from ..util import remove_whitespace
		return baseconv('b6d').tohex(remove_whitespace(b6d_str), pad=pad)



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
# Licensed under the GNU General Public License, Version 3:
#   https://www.gnu.org/licenses
# Public project repositories:
#   https://github.com/mmgen/mmgen-wallet
#   https://gitlab.com/mmgen/mmgen-wallet

"""
proto.eth.tx.info: Ethereum transaction info class
"""

from ....tx.info import TxInfo
from ....color import red, yellow, blue, cyan, pink
from ....obj import NonNegativeInt

from ...vm.tx.info import VmTxInfo, mmid_disp

class TxInfo(VmTxInfo, TxInfo):

	to_addr_key = 'to'

	def format_body(self, blockcount, nonmm_str, max_mmwid, enl, *, terse, sort):
		tx = self.tx
		t = tx.txobj
		td = t['data']
		to_addr = t[self.to_addr_key]
		tokenswap = tx.is_swap and tx.is_token
		fs = """
			From:      {f}{f_mmid}
			{toaddr}   {t}{t_mmid}{tvault}
			Amount:    {a} {c}
			Gas price: {g} Gwei
			Gas limit: {G}{G_dec}
			Nonce:     {n}
			Data:      {d}
		""".strip().replace('\t', '') + ('\nMemo:      {m}' if tx.is_swap else '')
		return fs.format(
			f      = t['from'].hl(0),
			t      = to_addr.hl(0) if to_addr else blue('None'),
			a      = t['amt'].hl(),
			toaddr = ('Router:' if tokenswap else 'Vault:' if tx.is_swap else 'To:').ljust(8),
			tvault = (f'\nVault:     {cyan(tx.token_vault_addr)}' if tokenswap else ''),
			n      = t['nonce'].hl(),
			d      = blue('None') if not td else '{}... ({} bytes)'.format(td[:40], len(td)//2),
			m      = pink(tx.swap_memo) if tx.is_swap else None,
			c      = tx.proto.dcoin if len(tx.outputs) else '',
			g      = yellow(tx.pretty_fmt_fee(t['gasPrice'].to_unit('Gwei'))),
			G      = NonNegativeInt(tx.total_gas).hl(),
			G_dec  = red(f" ({t['startGas']} + {t['router_gas']})") if tokenswap else '',
			f_mmid = mmid_disp(tx.inputs[0], nonmm_str),
			t_mmid = mmid_disp(tx.outputs[0], nonmm_str) if tx.outputs and not tx.is_swap else '') + '\n\n'

	def format_abs_fee(self, iwidth, /, *, color=None):
		return self.tx.fee.fmt(iwidth, color=color) + (' (max)' if self.tx.txobj['data'] else '')

	def format_rel_fee(self):
		return ' ({} of spend amount)'.format(
			pink('{:0.6f}%'.format(self.tx.fee / self.tx.send_amt * 100))
		)

	def format_verbose_footer(self):
		if self.tx.txobj['data'] and not self.tx.is_swap:
			from ....util import pp_fmt
			from ..contract import parse_abi
			return '\nParsed contract data: ' + pp_fmt(parse_abi(self.tx.txobj['data']))
		else:
			return ''

class TokenTxInfo(TxInfo):
	to_addr_key = 'token_to'

	def format_rel_fee(self):
		return ''

	def format_body(self, *args, **kwargs):
		return 'Token:     {d} {c}\n{r}'.format(
			d = self.tx.txobj['token_addr'].hl(0),
			c = blue('(' + self.tx.proto.dcoin + ')'),
			r = super().format_body(*args, **kwargs))



#!/usr/bin/env python3
#
# mmgen = Multi-Mode GENerator, command-line Bitcoin cold storage solution
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ------------------------------------------------------------------------------
# MMGen note: this is a sorted version of the wordlist. The original can be
# found in the file 'wordlists/mn_wordlist.c' of the mmgen distribution.
# ------------------------------------------------------------------------------
# Oren Tirosh is no longer maintaining the original version of this
# project.
#
# Stephen Paul Weber likes it and is making it accessible on GitHub.
# https://github.com/singpolyma/mnemonicode
# ------------------------------------------------------------------------------
# Original copyright notice follows:
# ------------------------------------------------------------------------------
# Oren Tirosh wordlist
# Copyright (c) 2000  Oren Tirosh <oren@hishome.net>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

words = tuple("""
abraham
absent
absorb
academy
accent
acid
acrobat
action
active
actor
adam
address
adios
admiral
adrian
africa
agatha
agenda
agent
airline
airport
alabama
aladdin
alamo
alarm
alaska
albert
albino
album
alcohol
alert
alex
alfonso
alfred
algebra
alias
alibi
alice
alien
almanac
almond
aloha
alpha
alpine
amadeus
amanda
amazon
amber
ambient
amen
america
amigo
ammonia
analog
analyze
anatomy
andrea
andy
angel
animal
anita
annual
answer
antenna
antonio
anvil
apollo
appear
apple
april
apropos
arcade
archer
archive
arctic
arena
ariel
arizona
armada
armor
arnold
aroma
arrow
arsenal
arthur
artist
asia
aspect
aspirin
athena
athlete
atlanta
atlas
atomic
audio
august
aurora
austin
austria
avalon
avatar
avenue
average
axiom
axis
aztec
baboon
baby
bagel
baggage
bahama
baker
balance
bali
ballad
ballet
balloon
balsa
bambino
bamboo
banana
bandit
banjo
bank
barbara
barcode
baron
basic
basil
basket
battery
bazaar
bazooka
beach
beast
beatles
beauty
before
begin
belgium
benefit
benny
berlin
bermuda
bernard
betty
between
beyond
bicycle
bikini
billy
binary
bingo
biology
biscuit
bishop
bison
blast
bless
blitz
block
blonde
blue
bogart
bombay
bonanza
bonjour
bonus
book
border
boris
boston
bottle
boxer
brain
brandy
brave
bravo
brazil
bread
break
brenda
bridge
brigade
british
broken
bronze
brother
brown
bruce
bruno
brush
bucket
budget
buenos
buffalo
bundle
burger
burma
button
buzzer
byte
cabaret
cabinet
cable
cactus
cadet
caesar
cafe
cairo
cake
calypso
camel
camera
camilla
campus
canada
canal
canary
candid
candle
cannon
canoe
cantina
canvas
canyon
capital
capitan
capsule
caramel
caravan
carbon
career
cargo
carlo
carmen
carol
carpet
carrot
cartel
cartoon
casino
castle
castro
catalog
cave
caviar
cecilia
cello
celtic
cement
center
century
ceramic
chamber
chance
change
channel
chant
chaos
chapter
chariot
charlie
charm
charter
cheese
chef
chemist
cherry
chess
chicago
chicken
chief
child
china
choice
chris
chrome
cigar
cinema
cipher
circle
circus
citizen
citrus
city
civil
clara
clarion
clark
classic
claudia
clean
clever
client
cliff
climax
clinic
clock
clone
cloud
club
cobalt
cobra
cockpit
coconut
cola
collect
college
colombo
colony
color
combat
comedy
comet
command
common
compact
company
compare
compass
complex
comrade
conan
concept
concert
condor
conduct
congo
connect
consul
contact
content
context
contour
control
convert
cool
copper
copy
coral
corner
corona
correct
cosmos
costume
cotton
couple
courage
cover
cowboy
crack
craft
crash
crater
credit
cricket
crimson
critic
crown
crystal
cuba
cubic
culture
cupid
current
cycle
cyclone
dallas
dance
daniel
danube
darwin
data
david
deal
decade
decide
decimal
declare
degree
delete
deliver
delphi
delta
deluxe
demand
demo
denmark
denver
depend
derby
desert
design
desire
detail
detect
develop
dexter
diagram
dialog
diamond
diana
diego
diesel
diet
digital
dilemma
dinner
diploma
direct
disco
disney
dispute
distant
divide
doctor
dolby
dollar
domain
domingo
dominic
domino
donald
donor
door
double
dragon
drama
dream
drink
driver
druid
drum
dublin
duet
dynamic
dynasty
eagle
earth
east
easy
echo
eclipse
ecology
economy
eddie
edgar
edison
edition
editor
educate
edward
effect
ego
egypt
elastic
electra
elegant
element
elite
elvis
email
emerald
emotion
empire
empty
energy
engine
english
enigma
enjoy
enrico
episode
epoxy
equal
equator
eric
erosion
escape
escort
eternal
ethnic
europe
evening
event
everest
evident
evita
exact
example
except
exhibit
exile
exit
exodus
exotic
expand
explain
explore
export
express
extend
extra
extreme
fabric
factor
falcon
fame
family
famous
fantasy
farmer
fashion
fast
father
fax
felix
ferrari
fiber
fiction
fidel
field
fiesta
figure
film
filter
final
finance
finish
finland
fiona
fire
first
fish
flag
flame
flash
flex
flipper
float
flood
floor
florida
flower
fluid
flute
focus
folio
food
forbid
ford
forest
forever
forget
formal
format
formula
fortune
forum
forward
fossil
fractal
fragile
frame
france
frank
freddie
freedom
fresh
friday
friend
frog
front
frozen
fruit
fuel
fuji
future
gabriel
galaxy
galileo
gallery
gallop
game
gamma
garage
garbo
garcia
garden
garlic
gate
gemini
general
genesis
genetic
geneva
genius
gentle
george
germany
giant
gibson
gilbert
ginger
giraffe
gizmo
glass
global
gloria
goblin
gold
golf
gondola
gong
good
gopher
gordon
gorilla
gossip
grace
gram
grand
granite
graph
gravity
gray
greek
green
gregory
grid
griffin
grille
ground
group
guest
guide
guitar
guru
gustav
gyro
habitat
hair
halt
hamlet
hammer
hand
happy
harbor
harlem
harmony
harris
harvard
harvest
havana
hawaii
hazard
heart
heaven
heavy
helena
helium
hello
henry
herbert
herman
heroic
hexagon
hilton
hippie
history
hobby
holiday
honey
hope
horizon
horse
hostel
hotel
house
human
humor
hunter
husband
hydro
ibiza
iceberg
icon
idea
igloo
igor
image
imagine
imitate
immune
impact
import
inca
inch
index
india
indigo
infant
info
ingrid
initial
input
insect
inside
instant
invent
invest
invite
iris
iron
isabel
isotope
italian
ivan
ivory
jacket
jackson
jacob
jaguar
jamaica
james
janet
japan
jargon
jasmine
jason
java
jazz
jeep
jerome
jessica
jester
jet
jimmy
job
joel
john
join
joker
jordan
joseph
joshua
journal
judge
judo
juice
juliet
julius
july
jumbo
jump
june
jungle
junior
jupiter
justice
justin
kansas
karate
karl
karma
kayak
kermit
kevin
kilo
kimono
kinetic
king
kitchen
kiwi
koala
korea
labor
ladder
lady
lagoon
lake
laptop
laser
latin
laura
lava
lazarus
learn
lecture
left
legacy
legal
legend
lemon
leonid
lesson
letter
level
lexicon
liberal
libra
license
life
light
lima
limbo
limit
linda
linear
lion
liquid
list
liter
lithium
little
llama
lobby
lobster
local
locate
logic
logo
lola
london
lopez
lorenzo
lotus
love
loyal
lucas
lucky
lunar
lunch
machine
macro
madam
madonna
madrid
maestro
magenta
magic
magnet
magnum
mailbox
major
malta
mama
mambo
mammal
manager
mango
manila
manual
marble
marco
margo
marina
marion
market
mars
martin
marvin
mary
mask
master
match
matrix
maximum
maxwell
mayday
mayor
maze
meaning
media
medical
medusa
mega
melody
melon
member
memo
memphis
mental
mentor
menu
mercury
mercy
message
metal
meteor
meter
method
metro
mexico
miami
michael
micro
middle
miguel
mike
milan
mile
milk
miller
million
mimic
mimosa
mineral
minimum
minus
minute
miracle
mirage
miranda
mirror
mission
mister
mixer
mobile
model
modem
modern
modest
modular
moment
monaco
monarch
monday
money
monica
monitor
monkey
mono
monster
montana
moral
morgan
morning
morph
morris
moses
motel
mother
motif
motor
mouse
mozart
multi
museum
music
mustang
mystery
nadia
nancy
natasha
native
nato
natural
navy
nebula
nectar
needle
nelson
neon
nepal
neptune
nerve
network
neuron
neutral
nevada
never
news
newton
next
nice
nickel
night
nikita
nina
ninja
nirvana
nissan
nitro
nixon
nobel
nobody
noise
nominal
normal
north
norway
nothing
nova
novel
nuclear
null
number
numeric
nurse
nylon
oasis
oberon
object
observe
ocean
octavia
october
octopus
office
ohio
olga
oliver
olivia
olympic
omega
open
opera
opinion
optic
optimal
option
opus
orange
orbit
orca
orchid
order
oregano
organic
orient
origami
origin
orinoco
orion
orlando
oscar
othello
outside
oval
owner
oxford
oxygen
ozone
pablo
pacific
package
page
pagoda
paint
palace
palma
pamela
panama
pancake
panda
pandora
panel
panic
panther
papa
paper
paprika
parade
paradox
pardon
parent
paris
parker
parking
parody
parole
partner
passage
passive
pasta
pastel
patent
patient
patriot
patrol
patron
pattern
paul
peace
pearl
pedro
pegasus
pelican
pencil
penguin
people
pepper
percent
perfect
perform
perfume
period
permit
person
peru
phantom
philips
phoenix
phone
photo
phrase
piano
picasso
picnic
picture
pierre
pigment
pilgrim
pilot
pinball
pioneer
pirate
pixel
pizza
place
planet
plasma
plaster
plastic
plate
plato
plaza
plume
pluto
pocket
podium
poem
poetic
pogo
point
poker
polaris
police
polite
politic
polka
polo
polygon
poncho
pony
popcorn
popular
portal
postage
postal
potato
powder
prague
precise
prefix
prelude
premium
prepare
present
press
presto
pretend
pretty
price
prime
prince
printer
prism
private
prize
process
product
profile
profit
program
project
promise
promo
protect
protein
proton
provide
proxy
public
pulse
puma
pump
pupil
puzzle
pyramid
python
quality
quarter
quasi
quebec
queen
quest
quick
quiet
quiz
quota
rabbit
race
rachel
radar
radical
radio
radius
rainbow
raja
ralph
ramirez
random
ranger
rapid
ravioli
raymond
rebel
record
recycle
reflex
reform
regard
region
regular
relax
remark
remote
rent
repair
reply
report
reptile
reserve
respect
respond
result
resume
retro
reunion
reverse
reward
rhino
ribbon
ricardo
richard
rider
right
ringo
rio
risk
ritual
rival
river
riviera
road
robert
robin
robot
rocket
rodent
rodeo
roger
roman
romeo
rondo
roof
rose
round
rover
royal
rubber
ruby
rudolf
rufus
russian
sabine
sabrina
saddle
safari
saga
sahara
sailor
saint
salad
salami
salary
salmon
salon
salsa
salt
salute
samba
sample
samuel
sandra
santana
sardine
satire
saturn
savage
scale
scarlet
scholar
school
scoop
scorpio
scratch
screen
script
scroll
scuba
season
second
secret
section
sector
secure
segment
select
seminar
senator
senior
sensor
serial
serpent
service
shadow
shake
shallow
shampoo
shannon
sharon
sharp
shave
shelf
shelter
sheriff
sherman
shine
ship
shirt
shock
shoe
short
shrink
side
sierra
sigma
signal
silence
silicon
silk
silver
similar
simon
simple
sinatra
sincere
singer
single
siren
sister
size
ski
slalom
slang
sleep
slogan
slow
small
smart
smile
smoke
snake
snow
social
society
soda
sofia
solar
solid
solo
sonar
sonata
song
sonic
soprano
sound
source
south
soviet
spain
spark
sparta
special
speech
speed
spell
spend
sphere
spider
spiral
spirit
split
sponsor
spoon
sport
spray
spring
square
stadium
stage
stamp
stand
star
state
static
station
status
stella
stereo
stick
sting
stock
stone
stop
store
storm
story
strange
street
stretch
strong
stuart
student
studio
style
subject
subway
sugar
sulfur
sultan
summer
sunday
sunset
super
support
survive
susan
sushi
suzuki
sweden
sweet
swim
swing
switch
symbol
system
table
taboo
tactic
tahiti
talent
tango
tape
target
tarzan
taxi
teacher
telecom
telex
temple
tempo
tennis
texas
textile
theory
thermos
think
thomas
tibet
ticket
tictac
tiger
time
tina
titanic
toast
tobacco
today
toga
tokyo
tomato
tommy
tonight
topic
torch
tornado
toronto
torpedo
torso
total
totem
touch
tourist
tower
toyota
tractor
trade
traffic
transit
trapeze
travel
tribal
tribune
trick
trident
trilogy
trinity
tripod
triton
trivial
tropic
truck
trumpet
trust
tulip
tuna
tunnel
turbo
turtle
twin
twist
type
ultra
uncle
under
unicorn
uniform
union
unique
unit
update
uranium
urban
urgent
user
vacuum
valery
valid
value
vampire
vanilla
vatican
vega
velvet
vendor
venice
ventura
venus
verbal
verona
version
vertigo
veteran
vibrate
victor
video
vienna
viking
village
vincent
violet
violin
virgo
virtual
virus
visa
visible
vision
visitor
vista
visual
vital
vitamin
viva
vocal
vodka
voice
volcano
voltage
volume
voodoo
vortex
voyage
waiter
warning
watch
water
wave
weather
wedding
weekend
welcome
western
wheel
whiskey
william
window
winter
wisdom
wizard
wolf
wonder
world
xray
yankee
year
yellow
yes
yoga
yogurt
young
yoyo
zebra
zero
zigzag
zipper
zodiac
zoom
""".split())



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
# Licensed under the GNU General Public License, Version 3:
#   https://www.gnu.org/licenses
# Public project repositories:
#   https://github.com/mmgen/mmgen-wallet
#   https://gitlab.com/mmgen/mmgen-wallet

"""
help.txsign: txsign help notes for the MMGen Wallet suite
"""

from ..cfg import gc
from ..daemon import CoinDaemon

def help(proto, cfg):

	def coind_exec():
		return CoinDaemon(cfg, network_id=proto.coin).exec_fn if proto.coin in CoinDaemon.coins else 'bitcoind'

	return """
Transactions may contain both {pnm} or non-{pnm} input addresses.

To sign non-{pnm} inputs, a coin daemon wallet dump or flat key list is used
as the key source (--keys-from-file option).

To sign {pnm} inputs, key data is generated from a seed as with the
{pnl}-addrgen and {pnl}-keygen commands.  Alternatively, a key-address file
may be used (--mmgen-keys-from-file option).

Multiple wallets or other seed files can be listed on the command line in
any order.  If the seeds required to sign the transaction’s inputs are not
found in these files (or in the default wallet), the user will be prompted
for seed data interactively.

To prevent an attacker from crafting transactions with bogus {pnm}-to-{pnu}
address mappings, all outputs to {pnm} addresses are verified with a seed
source.  Therefore, seed files or a key-address file for all {pnm} outputs
must also be supplied on the command line if the data can’t be found in the
default wallet.
""".format(
	pnm = gc.proj_name,
	pnu = proto.name,
	pnl = gc.proj_name.lower())



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
# Licensed under the GNU General Public License, Version 3:
#   https://www.gnu.org/licenses
# Public project repositories:
#   https://github.com/mmgen/mmgen-wallet
#   https://gitlab.com/mmgen/mmgen-wallet

"""
proto.eth.tw.rpc: Ethereum base protocol tracking wallet RPC class
"""

from ....tw.rpc import TwRPC

class EthereumTwRPC(TwRPC):
	pass

class EthereumTokenTwRPC(EthereumTwRPC):
	pass



#!/usr/bin/env python3

"""
test.modtest_d.seedsplit: seed splitting unit test for the MMGen suite
"""

from mmgen.util import msg, msg_r

from ..include.common import cfg, vmsg, vmsg_r

class unit_test:

	def run_test(self, name, ut):
		from mmgen.seed import Seed
		from mmgen.seedsplit import SeedShareList, SeedShareIdx

		cfg.debug_subseed = bool(cfg.verbose)

		def basic_ops(master_idx):
			test_data = {
				'default': (
					(8, '4710FBF0', 'B3D9411B', '2670E83D', 'D1FC57ED', 'AE49CABE', '63FFBA62', 0, 0),
					(6, '9D07ABBD', 'AF5DC2F6', '1A3BBDAC', '2548AEE9', 'B94F7450', '1F4E5A12', 0, 0),
					(4, '43670520', '1F72C066', 'E5AA8DA1', 'A33966A0', 'D2BCE0A5', 'A568C315', 0, 0),
				),
				'φυβαρ': (
					(8, '4710FBF0', '269D658C', '9D25889E', '6D730ECB', 'C61A963F', '9FE99C05', 0, 0),
					(6, '9D07ABBD', '4998B33E', 'F00CE041', 'C612BEE5', '35CD3675', '41B3BE61', 0, 0),
					(4, '43670520', '77140076', 'EA82CB30', '80F7AEDE', 'D168D768', '77BE57AA', 0, 0),
				)
			}
			test_data_master = {
				'1': {
					'default': (
						(8, '4710FBF0', '6AE6177F', 'AC12090C', '6AE6177F',
							'3E87A907', '7D1FEA56', 'BFEBFFFF', '629A9808'),
						(4, '43670520', '6739535C', 'ABF4DD38', '6739535C',
							'778E9C60', '89CBCFD2', '689FABF5', '70BED76B'),
					),
					'φυβαρ': (
						(8, '4710FBF0', '6AE6177F', 'AC5FA32E', '6AE6177F',
							'9777A750', 'C7CF2AFC', '035AAACB', 'C777FBE4'),
						(4, '43670520', '6739535C', '37EBA2F5', '6739535C',
							'927549D2', '29BADEE7', '9CA73A03', '313F5528'))
				},
				'5': {
					'default': (
						(8, '4710FBF0', '5EFAC3D6', 'B489167D', '5EFAC3D6',
							'BB004DC5', '1A0381C0', '4EA182E3', '547FB2DC'),
						(4, '43670520', 'EE93DB0E', '44962A7D', 'EE93DB0E',
							'07339882', '376A05B1', 'CE51D022', '00149CA3'),
					),
					'φυβαρ': (
						(8, '4710FBF0', '5EFAC3D6', 'A6E27EE3', '5EFAC3D6',
							'32C24668', 'B4C54297', '1EC9B71B', '8C5C6B1C'),
						(4, '43670520', 'EE93DB0E', 'B584E963', 'EE93DB0E',
							'4BEA2AB2', '4BEA65C7', '140FC43F', 'BBD19461'))
				}
			}
			if master_idx:
				test_data = test_data_master[str(master_idx)]

			for id_str in (None, 'default', 'φυβαρ'):
				msg_r(f'Testing basic ops (id_str={id_str!r}, master_idx={master_idx})...')
				vmsg('')

				for a, b, c, d, e, f, h, i, p in test_data[id_str if id_str is not None else 'default']:
					seed_bin = bytes.fromhex('deadbeef' * a)
					seed = Seed(cfg, seed_bin=seed_bin)
					assert seed.sid == b, seed.sid

					for share_count, j, k, l, m in (
							(2, c, c, d, i),
							(5, e, f, h, p)):

						shares = seed.split(share_count, id_str=id_str, master_idx=master_idx)
						A = len(shares)
						assert A == share_count, A

						s = shares.format()
						vmsg_r(f'\n{s}')
						assert len(s.strip().split('\n')) == share_count+6, s

						if master_idx:
							A = shares.get_share_by_idx(1, base_seed=False).sid
							B = shares.get_share_by_seed_id(j, base_seed=False).sid
							assert A == B == m, A

						A = shares.get_share_by_idx(1, base_seed=True).sid
						B = shares.get_share_by_seed_id(j, base_seed=True).sid
						assert A == B == j, A

						A = shares.get_share_by_idx(share_count-1, base_seed=True).sid
						B = shares.get_share_by_seed_id(k, base_seed=True).sid
						assert A == B == k, A

						A = shares.get_share_by_idx(share_count).sid
						B = shares.get_share_by_seed_id(l).sid
						assert A == B == l, A

						A = shares.join().sid
						assert A == b, A

						if master_idx:
							slist = [shares.get_share_by_idx(i+1, base_seed=True) for i in range(len(shares))]
							A = Seed.join_shares(cfg, slist, master_idx=master_idx, id_str=id_str).sid
							assert A == b, A

				msg('OK')

		def defaults_and_limits():
			msg_r('Testing defaults and limits...')

			seed_bin = bytes.fromhex('deadbeef' * 8)
			seed = Seed(cfg, seed_bin=seed_bin)

			shares = seed.split(SeedShareIdx.max_val)
			s = shares.format()
#			vmsg_r(f'\n{s}')
			assert len(s.strip().split('\n')) == 1030, s

			A = shares.get_share_by_idx(1024).sid
			B = shares.get_share_by_seed_id('4BA23728').sid
			assert A == '4BA23728', A
			assert B == '4BA23728', B

			A = shares.join().sid
			B = seed.sid
			assert A == B, A

			msg('OK')

		def collisions(seed_hex, ss_count, last_sid, collisions_chk, master_idx):

			msg_r(f'Testing Seed ID collisions ({ss_count} seed shares, master_idx={master_idx})...')
			vmsg('')

			seed_bin = bytes.fromhex(seed_hex)
			seed = Seed(cfg, seed_bin=seed_bin)

			SeedShareIdx.max_val = ss_count
			shares = seed.split(ss_count, master_idx=master_idx)
			A = shares.get_share_by_idx(ss_count).sid
			B = shares.get_share_by_seed_id(last_sid).sid
			assert A == last_sid, A
			assert B == last_sid, B

			assert shares.nonce_start == 0, shares.nonce_start

			collisions = 0
			for sid in shares.data['long']:
				collisions += shares.data['long'][sid][1]

			assert collisions == collisions_chk, collisions
			vmsg_r(f'{collisions} collisions, last_sid {last_sid}')
			msg('OK')

		def last_share_collisions():
			msg_r('Testing last share collisions with shortened Seed IDs')
			vmsg('')
			seed_bin = bytes.fromhex('2eadbeef'*8)
			seed = Seed(cfg, seed_bin=seed_bin)
			ssm_save = SeedShareIdx.max_val
			ssm = SeedShareIdx.max_val = 2048
			shares = SeedShareList(seed, count=ssm, id_str='foo', master_idx=1, debug_last_share=True)
			lsid = shares.last_share.sid
			collisions = shares.data['long'][lsid][1]
			assert collisions == 2, collisions
			assert lsid == 'B5B8AD09', lsid
			SeedShareIdx.max_val = ssm_save
			vmsg_r(f'{collisions} collisions, last_share sid {lsid}')
			msg('..OK')

		basic_ops(master_idx=None)
		basic_ops(master_idx=1)
		basic_ops(master_idx=5)
		defaults_and_limits()
		last_share_collisions()
		collisions('1dabcdef'*4, 65535, 'B5CBCE0A', 3, master_idx=None)
		collisions('18abcdef'*4, 65535, 'FF03CE82', 3, master_idx=1)

		return True



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
# Licensed under the GNU General Public License, Version 3:
#   https://www.gnu.org/licenses
# Public project repositories:
#   https://github.com/mmgen/mmgen-wallet
#   https://gitlab.com/mmgen/mmgen-wallet

"""
proto.zec.keygen: Zcash-Z public key generation backend for the MMGen suite
"""

from ...key import PubKey
from ...keygen import keygen_base

class backend:

	class nacl(keygen_base):

		production_safe = True

		def __init__(self, cfg):
			super().__init__(cfg)
			from nacl.bindings import crypto_scalarmult_base
			self.crypto_scalarmult_base = crypto_scalarmult_base
			from ...sha2 import Sha256
			self.Sha256 = Sha256

		def zhash256(self, s, t):
			s = bytearray(s + bytes(32))
			s[0] |= 0xc0
			s[32] = t
			return self.Sha256(s, preprocess=False).digest()

		def to_pubkey(self, privkey):
			return PubKey(
				self.zhash256(privkey, 0)
				+ self.crypto_scalarmult_base(self.zhash256(privkey, 1)),
				compressed = privkey.compressed
			)

		def to_viewkey(self, privkey):
			vk = bytearray(self.zhash256(privkey, 0) + self.zhash256(privkey, 1))
			vk[32] &= 0xf8
			vk[63] &= 0x7f
			vk[63] |= 0x40
			return vk



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
# Licensed under the GNU General Public License, Version 3:
#   https://www.gnu.org/licenses
# Public project repositories:
#   https://github.com/mmgen/mmgen-wallet
#   https://gitlab.com/mmgen/mmgen-wallet

"""
test.cmdtest_d.automount: autosigning with automount tests for the cmdtest.py test suite
"""
import time

from .autosign import CmdTestAutosignThreaded
from .regtest import CmdTestRegtest, rt_pw
from ..include.common import gr_uc, create_addrpairs

class CmdTestAutosignAutomount(CmdTestAutosignThreaded, CmdTestRegtest):
	'automounted transacting operations via regtest mode'

	networks = ('btc', 'bch', 'ltc')
	tmpdir_nums = [49]
	bdb_wallet = True
	keylist_passwd = 'abc'

	rt_data = {
		'rtFundAmt': {'btc':'500', 'bch':'500', 'ltc':'5500'},
	}
	bal1_chk = {
		'btc': '502.46',
		'bch': '502.46',
		'ltc': '5502.46'}
	bal2_chk = {
		'btc': '493.56992828',
		'bch': '501.22524576',
		'ltc': '5493.56992828'}

	cmd_group = (
		('setup',                            'regtest mode setup'),
		('walletgen_alice',                  'wallet generation (Alice)'),
		('addrgen_alice',                    'address generation (Alice)'),
		('addrimport_alice',                 'importing Alice’s addresses'),
		('addrimport_alice_non_mmgen',       'importing Alice’s non-MMGen addresses'),
		('fund_alice',                       'funding Alice’s wallet'),
		('fund_alice_non_mmgen1',            'funding Alice’s wallet (non-MMGen addr #1)'),
		('fund_alice_non_mmgen2',            'funding Alice’s wallet (non-MMGen addr #2)'),
		('generate',                         'mining a block'),
		('alice_bal1',                       'checking Alice’s balance'),
		('alice_txcreate1',                  'creating a transaction'),
		('alice_txcreate_bad_have_unsigned', 'creating the transaction again (error)'),
		('alice_run_autosign_setup',         'running ‘autosign setup’ (with default wallet)'),
		('wait_loop_start',                  'starting autosign wait loop'),
		('alice_txstatus1',                  'getting transaction status (unsigned)'),
		('alice_txstatus2',                  'getting transaction status (unsent)'),
		('alice_txcreate_bad_have_unsent',   'creating the transaction again (error)'),
		('alice_txsend1',                    'sending a transaction, editing comment'),
		('alice_txstatus3',                  'getting transaction status (in mempool)'),
		('alice_txsend_bad_no_unsent',       'sending the transaction again (error)'),
		('generate',                         'mining a block'),
		('alice_txstatus4',                  'getting transaction status (one confirmation)'),
		('alice_txcreate2',                  'creating a transaction'),
		('alice_txsend_abort1',              'aborting the transaction (raw only)'),
		('alice_txsend_abort2',              'aborting the transaction again (error)'),
		('alice_txcreate3',                  'creating a transaction'),
		('alice_txsend_abort3',              'aborting the transaction (user exit)'),
		('alice_txsend_abort4',              'aborting the transaction (raw + signed)'),
		('alice_txsend_abort5',              'aborting the transaction again (error)'),
		('generate',                         'mining a block'),
		('alice_txcreate4',                  'creating a transaction'),
		('alice_txbump1',                    'bumping the unsigned transaction (error)'),
		('alice_txbump2',                    'bumping the unsent transaction (error)'),
		('alice_txsend2_dump_hex',           'dumping the transaction to hex'),
		('alice_txsend2_cli',                'sending the transaction via cli'),
		('alice_txsend2_mark_sent',          'marking the transaction sent'),
		('alice_txbump3',                    'bumping the transaction'),
		('alice_txsend3',                    'sending the bumped transaction'),
		('alice_txbump4',                    'bumping the transaction (new outputs, fee too low)'),
		('alice_txbump_abort1',              'aborting the transaction'),
		('alice_txbump5',                    'bumping the transaction (new outputs)'),
		('alice_txsend5',                    'sending the bumped transaction'),
		('alice_txstatus5',                  'getting transaction status (in mempool)'),
		('generate',                         'mining a block'),
		('alice_bal2',                       'checking Alice’s balance'),
		('wait_loop_kill',                   'stopping autosign wait loop'),
		('stop',                             'stopping regtest daemon'),
		('txview',                           'viewing transactions'),
	)

	def __init__(self, cfg, trunner, cfgs, spawn):

		self.coins = [cfg.coin.lower()]

		CmdTestAutosignThreaded.__init__(self, cfg, trunner, cfgs, spawn)
		CmdTestRegtest.__init__(self, cfg, trunner, cfgs, spawn)

		if trunner is None:
			return

		self.opts.append('--alice')

		self.non_mmgen_addrs = create_addrpairs(self.proto, 'C', 2)

	def addrimport_alice_non_mmgen(self):
		self.write_to_tmpfile(
			'non_mmgen_addrs',
			'\n'.join(e.addr for e in self.non_mmgen_addrs))
		return self.spawn(
			'mmgen-addrimport',
			['--alice', '--quiet', '--addrlist', f'{self.tmpdir}/non_mmgen_addrs'])

	def fund_alice_non_mmgen1(self):
		return self.fund_wallet('alice', '1.23', addr=self.non_mmgen_addrs[0].addr)

	def fund_alice_non_mmgen2(self):
		return self.fund_wallet('alice', '1.23', addr=self.non_mmgen_addrs[1].addr)

	def alice_bal1(self):
		return self._user_bal_cli('alice', chk=self.bal1_chk[self.coin])

	def alice_txcreate1(self):
		return self._user_txcreate(
			'alice',
			inputs = '1-3',
			tweaks = ['confirm_non_mmgen'],
			chg_addr = 'C:5',
			data_arg = 'data:'+gr_uc[:24])

	def alice_txcreate2(self):
		return self._user_txcreate('alice', chg_addr='L:5')

	alice_txcreate3 = alice_txcreate2

	def alice_txcreate4(self):
		return self._user_txcreate('alice', chg_addr='L:4', need_rbf=True)

	def _alice_txsend_abort(self, err=False, send_resp='y', expect=None, shred_expect=[]):
		self.insert_device_online()
		t = self.spawn(
				'mmgen-txsend',
				['--quiet', '--abort'],
				no_passthru_opts = ['coin'],
				exit_val = 2 if err else 1 if send_resp == 'n' else None)
		if err:
			t.expect(expect)
		else:
			t.expect('(y/N): ', send_resp)
			if expect:
				t.expect(expect)
			for pat in shred_expect:
				t.expect(pat, regex=True)
		t.read()
		self.remove_device_online()
		return t

	def alice_txsend_abort1(self):
		return self._alice_txsend_abort(shred_expect=['Shredding .*arawtx'])

	def alice_txsend_abort2(self):
		return self._alice_txsend_abort(err=True, expect='No unsent transactions')

	def alice_txsend_abort3(self):
		return self._alice_txsend_abort(send_resp='n', expect='Exiting at user request')

	def alice_txsend_abort4(self):
		self._wait_signed('transaction')
		return self._alice_txsend_abort(shred_expect=[r'Shredding .*arawtx', r'Shredding .*asigtx'])

	alice_txsend_abort5 = alice_txsend_abort2

	def alice_txcreate_bad_have_unsigned(self):
		return self._user_txcreate('alice', chg_addr='C:5', exit_val=2, expect_str='already present')

	def alice_txcreate_bad_have_unsent(self):
		return self._user_txcreate('alice', chg_addr='C:5', exit_val=2, expect_str='unsent transaction')

	def alice_run_autosign_setup(self):
		from mmgen.crypto import Crypto
		from mmgen.cfg import Config
		new_cfg = Config({'_clone': self.cfg, 'usr_randchars': 0, 'hash_preset': '1'})
		enc_data = Crypto(new_cfg).mmgen_encrypt(
			'\n'.join(e.wif for e in self.non_mmgen_addrs).encode(), passwd=self.keylist_passwd)
		self.write_to_tmpfile('non_mmgen_keys.mmenc', enc_data, binary=True)
		return self.run_setup(
			mn_type = 'default',
			use_dfl_wallet = True,
			wallet_passwd = rt_pw,
			add_opts = [f'--keys-from-file={self.tmpdir}/non_mmgen_keys.mmenc'],
			keylist_passwd = self.keylist_passwd)

	def alice_txsend1(self):
		return self._user_txsend('alice', comment='This one’s worth a comment', no_wait=True)

	def alice_txsend2_dump_hex(self):
		return self._user_txsend('alice', need_rbf=True, dump_hex=True)

	def alice_txsend2_cli(self):
		if not self.proto.cap('rbf'):
			return 'skip'
		return self._user_dump_hex_send_cli('alice')

	def alice_txsend2_mark_sent(self):
		return self._user_txsend('alice', need_rbf=True, mark_sent=True)

	def alice_txsend3(self):
		return self._user_txsend('alice', need_rbf=True)

	def alice_txsend5(self):
		return self._user_txsend('alice', need_rbf=True)

	def _alice_txstatus(self, expect, exit_val=None, need_rbf=False):

		if need_rbf and not self.proto.cap('rbf'):
			return 'skip'

		self.insert_device_online()
		t = self.spawn(
				'mmgen-txsend',
				['--alice', '--autosign', '--status', '--verbose'],
				no_passthru_opts = ['coin'],
				exit_val = exit_val)
		t.expect(expect)
		if not exit_val:
			t.expect('view: ', 'n')
		t.read()
		self.remove_device_online()
		return t

	def alice_txstatus1(self):
		return self._alice_txstatus('unsigned', 1)

	def alice_txstatus2(self):
		self._wait_signed('transaction')
		return self._alice_txstatus('unsent', 1)

	def alice_txstatus3(self):
		return self._alice_txstatus('in mempool', 0)

	def alice_txstatus4(self):
		return self._alice_txstatus('1 confirmation', 0)

	def alice_txstatus5(self):
		return self._alice_txstatus('in mempool', need_rbf=True)

	def alice_txsend_bad_no_unsent(self):
		self.insert_device_online()
		t = self.spawn('mmgen-txsend', ['--quiet', '--autosign'], exit_val=2, no_passthru_opts=['coin'])
		t.expect('No unsent transactions')
		t.read()
		self.remove_device_online()
		return t

	def _alice_txbump(self, fee_opt=None, output_args=[], bad_tx_expect=None, low_fee_fix=None):
		if not self.proto.cap('rbf'):
			return 'skip'
		self.insert_device_online()
		t = self.spawn(
				'mmgen-txbump',
				['--alice', '--autosign']
				+ ([fee_opt] if fee_opt else [])
				+ output_args,
				exit_val = 1 if bad_tx_expect else None)
		if bad_tx_expect:
			time.sleep(0.5)
			t.expect('Only sent transactions')
			t.expect(bad_tx_expect)
		else:
			if not output_args:
				t.expect(r'to deduct the fee from .* change output\): ', '\n', regex=True)
				t.expect(r'(Y/n): ', 'y')  # output OK?
			if low_fee_fix or not fee_opt:
				if low_fee_fix:
					t.expect('Please choose a higher fee')
				t.expect('transaction fee: ', (low_fee_fix or '200s') + '\n')
			if output_args:
				t.expect(r'(Y/n): ', 'y')
			t.expect(r'(Y/n): ', 'y')  # fee OK?
			t.expect(r'(y/N): ', '\n') # add comment?
			t.expect(r'(y/N): ', 'y')  # save?
		t.read()
		self.remove_device_online()
		return t

	def alice_txbump1(self):
		return self._alice_txbump(bad_tx_expect='unsigned transaction')

	def alice_txbump2(self):
		self._wait_signed('transaction')
		return self._alice_txbump(bad_tx_expect='unsent transaction')

	def alice_txbump3(self):
		return self._alice_txbump()

	def alice_txbump4(self):
		sid = self._user_sid('alice')
		return self._alice_txbump(
			fee_opt = '--fee=3s',
			output_args = [f'{self.burn_addr},7.654321', f'{sid}:C:1'],
			low_fee_fix = '300s')

	def alice_txbump_abort1(self):
		if not self.proto.cap('rbf'):
			return 'skip'
		return self._alice_txsend_abort(shred_expect=['Shredding .*arawtx'])

	def alice_txbump5(self):
		sid = self._user_sid('alice')
		return self._alice_txbump(
			fee_opt = '--fee=400s',
			output_args = ['data:message for posterity', f'{self.burn_addr},7.654321', f'{sid}:C:1'])

	def alice_bal2(self):
		return self.user_bal('alice', self.bal2_chk[self.coin])



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>

"""
test.objtest_d.common: shared data for MMGen data objects tests
"""

from ..include.common import getrand

r16, r32 = getrand(16), getrand(32)



#!/usr/bin/env python3

"""
test.modtest_d.lockable: unit test for the MMGen suite's Lockable class
"""

from decimal import Decimal
from mmgen.base_obj import AttrCtrl, Lockable

class unit_tests:

	def attrctl(self, name, ut, desc='AttrCtrl class'):

		class MyAttrCtrl(AttrCtrl):
			_autolock = False
			foo = 'fooval'
		ac = MyAttrCtrl()
		ac._lock()

		ac.foo = 'new fooval'
		ac.foo = 'new fooval2'

		class MyAttrCtrlAutolock(AttrCtrl):
			pass

		aca = MyAttrCtrlAutolock()

		class MyAttrCtrlClsCheck(AttrCtrl):
			_autolock = False
			_use_class_attr = True
			foo = 'fooval'
			bar = None
		acc = MyAttrCtrlClsCheck()
		acc._lock()

		acc.foo = 'new_fooval'
		acc.foo = 'new_fooval2'
		acc.bar = 'bar val'
		acc.bar = 1 # class attribute bar is None, so can be set to any type

		class MyAttrCtrlDflNone(AttrCtrl):
			_default_to_none = True
			foo = 'fooval'
			bar = None

		acdn = MyAttrCtrlDflNone()
		assert acdn.foo == 'fooval', f'{acdn.foo}'
		assert acdn.bar is None, f'{acdn.bar}'
		assert acdn.baz is None, f'{acdn.baz}'

		def bad1(): ac.x = 1
		def bad2(): acc.foo = 1
		def bad3(): aca._lock()
		def bad4(): acdn.baz = None

		ut.process_bad_data((
			('attr',            'AttributeError', 'has no attr', bad1),
			('attr type',       'AttributeError', 'type',        bad2),
			('call to _lock()', 'AssertionError', 'only once',   bad3),
			('attr',            'AttributeError', 'has no attr', bad4),
		))

		return True

	def base(self, name, ut, desc='Lockable class'):

		class MyLockable(Lockable): # class without attrs
			_autolock = False
			_set_ok = ('foo', 'baz', 'alpha', 'beta', 'gamma', 'delta', 'epsilon')
			_reset_ok = ('bar', 'baz')

		lc = MyLockable()
		lc.foo = None
		lc.bar = 'barval'
		lc.baz = 1
		lc.qux = 1

		# are these considered set?
		lc.alpha = 0             # yes
		lc.beta = False          # yes
		lc.gamma = Decimal('0')  # yes
		lc.delta = 0.0           # yes
		lc.epsilon = []          # no

		lc._lock()

		lc.foo = 'fooval2'
		lc.bar = 'barval2'
		lc.bar = 'barval3'
		lc.baz = 2
		lc.baz = 3

		lc.epsilon = [0]

		def bad1(): lc.foo = 'fooval3'
		def bad2(): lc.baz = 'str'
		def bad3(): lc.qux  = 2
		def bad4(): lc.x = 1
		def bad5(): lc.alpha = 0
		def bad6(): lc.beta = False
		def bad7(): lc.gamma = Decimal('0')
		def bad8(): lc.delta = float(0)
		def bad9(): lc.epsilon = [0]

		ut.process_bad_data((
			('attr (can’t reset)', 'AttributeError', 'reset',       bad1),
			('attr type (2)',      'AttributeError', 'type',        bad2),
			('attr (can’t set)',   'AttributeError', 'read-only',   bad3),
			('attr (2)',           'AttributeError', 'has no attr', bad4),
			('attr (can’t reset)', 'AttributeError', 'reset',       bad5),
			('attr (can’t reset)', 'AttributeError', 'reset',       bad6),
			('attr (can’t reset)', 'AttributeError', 'reset',       bad7),
			('attr (can’t reset)', 'AttributeError', 'reset',       bad8),
			('attr (can’t reset)', 'AttributeError', 'reset',       bad9),
		))

		return True

	def classattr(self, name, ut, desc='Lockable class with class attrs'):

		class MyLockableClsCheck(Lockable): # class with attrs
			_autolock = False
			_use_class_attr = True
			_set_ok = ('foo', 'baz')
			_reset_ok = ('bar', 'baz')
			foo = None
			bar = 1
			baz = 3.5
			qux = 'quxval'

		lcc = MyLockableClsCheck()
		lcc._lock()

		lcc.foo = 'fooval2' # class attribute foo is None, so can be set to any type
		lcc.bar = 2
		lcc.bar = 3   # bar is in reset list
		lcc.baz = 3.2
		lcc.baz = 3.1 # baz is in both lists

		def bad1(): lcc.bar = 'str'
		def bad2(): lcc.qux = 'quxval2'
		def bad3(): lcc.foo = 'fooval3'
		def bad4(): lcc.x = 1

		ut.process_bad_data((
			('attr type (3)',      'AttributeError', 'type',        bad1),
			('attr (can’t set)',   'AttributeError', 'read-only',   bad2),
			('attr (can’t reset)', 'AttributeError', 'reset',       bad3),
			('attr (3)',           'AttributeError', 'has no attr', bad4),
		))

		return True

	def autolock(self, name, ut, desc='Lockable class with autolock'):

		class MyLockableAutolock(Lockable):
			def __init__(self):
				self.foo = True

		lca = MyLockableAutolock()
		assert lca._autolock is True
		assert lca._locked is True
		assert lca.foo is True

		class MyLockableAutolockDflNone(Lockable):
			_default_to_none = True
			foo = 0

		lcdn = MyLockableAutolockDflNone()
		assert lcdn.foo == 0
		assert lcdn.bar is None

		class MyLockableBad(Lockable):
			_set_ok = ('foo', 'bar')
			foo = 1

		def bad1(): lca.foo = None
		def bad2(): MyLockableBad()
		def bad3(): lcdn.foo = 1
		def bad4(): lcdn.bar = None
		def bad5(): del lcdn.foo

		ut.process_bad_data((
			('attr (can’t set)',    'AttributeError', 'read-only',    bad1),
			('attr (bad _set_ok)',  'AssertionError', 'not found in', bad2),
			('attr (can’t set)',    'AttributeError', 'read-only',    bad3),
			('attr (5)',            'AttributeError', 'has no attr',  bad4),
			('attr (can’t delete)', 'AttributeError', 'not be delet', bad5),
		))

		return True



#!/usr/bin/env python3
#
# MMGen Wallet, a terminal-based cryptocurrency wallet
# Copyright (C)2013-2025 The MMGen Project <mmgen@tuta.io>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
test.cmdtest_d.include.common: Shared routines and data for the cmdtest.py test suite
"""

import sys, os

from mmgen.color import green, blue, gray
from mmgen.util import msg

from ...include.common import cfg, getrand, text_jp, text_zh, ascii_cyr_gr, lat_cyr_gr

rt_pw = 'abc-α'
ref_wallet_brainpass = 'abc'
ref_wallet_hash_preset = '1'
ref_wallet_incog_offset = 123

dfl_addr_idx_list = '1010,500-501,31-33,1,33,500,1011'
dfl_wpasswd = 'reference password'

pwfile = 'passwd_file'
hincog_fn = 'rand_data'
hincog_bytes = 1024*1024
hincog_offset = 98765
hincog_seedlen = 256

incog_id_fn = 'incog_id'
non_mmgen_fn = 'coinkey'

ref_dir = os.path.join('test', 'ref')

dfl_sid = '98831F3A'
dfl_words_file = os.path.join(ref_dir, '98831F3A.mmwords')
dfl_bip39_file = os.path.join(ref_dir, '98831F3A.bip39')

from mmgen.obj import MMGenTxComment, TwComment

tx_comment_jp = text_jp
tx_comment_zh = text_zh

lcg = ascii_cyr_gr if sys.platform == 'win32' else lat_cyr_gr # MSYS2 popen_spawn issue
tx_comment_lat_cyr_gr = lcg[:MMGenTxComment.max_len] # 72 chars

tw_comment_zh         = text_zh[:TwComment.max_screen_width // 2]
tw_comment_lat_cyr_gr = lcg[:TwComment.max_screen_width] # 80 chars

ref_bw_hash_preset = '1'
ref_bw_file = 'wallet.mmbrain'
ref_bw_file_spc = 'wallet-spaced.mmbrain'

ref_enc_fn = 'sample-text.mmenc'
tool_enc_passwd = "Scrypt it, don't hash it!"
chksum_pat = r'\b[A-F0-9]{4} [A-F0-9]{4} [A-F0-9]{4} [A-F0-9]{4}\b'

Ctrl_U = '\x15'

eth_inbound_addr = (28 * '0') + 'feedbeefcafe'

thorchain_router_addr_file = 'test/data_dir/thorchain_router_addr'

def ok_msg():
	if cfg.profile:
		return
	sys.stderr.write(green('\nOK\n') if cfg.exact_output or cfg.verbose else ' OK\n')

def skip(name, reason=None):
	msg(gray('Skipping {}{}'.format(name, f' ({reason})' if reason else '')))
	return 'skip'

def confirm_continue():
	from mmgen.ui import keypress_confirm
	if keypress_confirm(
			cfg,
			blue('Continue? (Y/n): '),
			default_yes     = True,
			complete_prompt = True):
		if cfg.verbose or cfg.exact_output:
			sys.stderr.write('\n')
	else:
		raise KeyboardInterrupt('Exiting at user request')

def randbool():
	return getrand(1).hex()[0] in '02468ace'

def cleanup_env(cfg):
	ret = dict(os.environ)
	if cfg.debug_utf8:
		return ret
	for k in cfg._env_opts:
		if k[:11] == 'MMGEN_DEBUG' and k in ret:
			del ret[k]
	return ret | {'EXEC_WRAPPER_DO_RUNTIME_MSG': ''}

def get_file_with_ext(
		tdir,
		ext,
		delete      = True,
		no_dot      = False,
		return_list = False,
		delete_all  = False,
		subdir      = None,
		substr      = False):

	dot = '' if no_dot else '.'

	def have_match(fn):
		return (
			fn == ext
			or fn.endswith(dot + ext)
			or (substr and ext in fn))

	if subdir:
		tdir = os.path.join(tdir, subdir)

	# Don’t use os.scandir here - it returns broken paths under Windows/MSYS2
	flist = [os.path.join(tdir, name) for name in os.listdir(tdir) if have_match(name)]

	if not flist:
		return False

	if return_list:
		return flist

	if len(flist) > 1 or delete_all:
		if delete or delete_all:
			if (cfg.exact_output or cfg.verbose) and not cfg.quiet:
				if delete_all:
					msg(f'Deleting all *{dot}{ext} files in ‘{tdir}’')
				else:
					msg(f'Multiple *{dot}{ext} files in ‘{tdir}’ - deleting')
			for f in flist:
				os.unlink(f)
		return False
	else:
		return flist[0]

def get_comment(do_shuffle=False):
	labels = [
		'Automotive',
		'Travel expenses',
		'Healthcare',
		tx_comment_jp[:40],
		tx_comment_zh[:40],
		'Alice’s allowance',
		'Bob’s bequest',
		'House purchase',
		'Real estate fund',
		'Job 1',
		'XYZ Corp.',
		'Eddie’s endowment',
		'Emergency fund',
		'Real estate fund',
		'Ian’s inheritance',
		'',
		'Rainy day',
		'Fred’s funds',
		'Job 2',
		'Carl’s capital',
	]
	from random import shuffle
	global label_iter
	try:
		return next(label_iter)
	except:
		if do_shuffle:
			shuffle(labels)
		label_iter = iter(labels)
		return next(label_iter)
